@@ -225,3 +225,26 @@ ods_hash_table_get_keys(GHashTable *table)
 
 	return list_keys;
 }
+
+gchar**
+ods_hash_table_keys2strv (GHashTable *list)
+{
+	guint list_size, i = 0;
+	GHashTableIter iter;
+	gpointer key, value;
+	gchar **list_keys = NULL;
+
+	list_size = g_hash_table_size (list);
+	if (list_size) {
+		list_keys = malloc (sizeof(gchar*) * (list_size+1));
+		g_hash_table_iter_init (&iter, list);
+		while (g_hash_table_iter_next (&iter, &key, &value))
+		{
+			list_keys[i] = g_strdup (key);
+			i++;
+		}
+		list_keys[i] = NULL;
+	}
+
+	return list_keys;
+}
