@@ -72,9 +72,17 @@ ods_error_get_type (void)
 			ENUM_ENTRY (ODS_ERROR_NOT_STARTED, "NotStarted"),
 			ENUM_ENTRY (ODS_ERROR_TRANSPORT_NOT_AVAILABLE, "TransportNotAvailable"),
 			ENUM_ENTRY (ODS_ERROR_CONNECTION_REFUSED, "ConnectionRefused"),
+			ENUM_ENTRY (ODS_ERROR_CONNECTION_TIMEOUT, "ConnectionTimeout"),
 			ENUM_ENTRY (ODS_ERROR_BAD_REQUEST, "BadRequest"),
 			ENUM_ENTRY (ODS_ERROR_NOT_IMPLEMENTED, "NotImplemented"),
 			ENUM_ENTRY (ODS_ERROR_SERVER_ERROR, "ServerError"),
+			ENUM_ENTRY (ODS_ERROR_TIMEOUT, "Timeout"),
+			ENUM_ENTRY (ODS_ERROR_CANCELLED, "Cancelled"),
+			ENUM_ENTRY (ODS_ERROR_NOT_MODIFIED, "NotModified"),
+			ENUM_ENTRY (ODS_ERROR_DATABASE_ERROR, "DatabaseError"),
+			ENUM_ENTRY (ODS_ERROR_CONFLICT, "Conflict"),
+			ENUM_ENTRY (ODS_ERROR_UNSUPPORTED_MEDIA_TYPE, "UnsupportedMediaType"),
+			ENUM_ENTRY (ODS_ERROR_NO_FIFO_READER, "NoFifoReader"),
 			{ 0, 0, 0 }
 		};
 		etype = g_enum_register_static ("OdsError", values);
@@ -91,6 +99,10 @@ ods_error_err2gerror (gint err, GError **error)
 		err = -err;
 
 	switch (err) {
+		case EACCES:
+			g_set_error (error, ODS_ERROR, ODS_ERROR_NOT_AUTHORIZED,
+						"Permission denied");
+			break;
 		case EBUSY:
 			g_set_error (error, ODS_ERROR, ODS_ERROR_BUSY,
 						"Another operation in progress");
@@ -107,6 +119,18 @@ ods_error_err2gerror (gint err, GError **error)
 			g_set_error (error, ODS_ERROR, ODS_ERROR_OUT_OF_MEMORY,
 						"Out of memory");
 			break;
+		case EPERM:
+			g_set_error (error, ODS_ERROR, ODS_ERROR_FORBIDDEN,
+						"Operation forbidden");
+			break;
+		case ENOSPC:
+			g_set_error (error, ODS_ERROR, ODS_ERROR_DATABASE_ERROR,
+						"No space left on device");
+			break;
+		case ENXIO:
+			g_set_error (error, ODS_ERROR, ODS_ERROR_NO_FIFO_READER,
+						"No process has opened supplied FIFO for reading");
+			break;
 		default:
 			g_set_error (error, ODS_ERROR, ODS_ERROR_FAILED,
 						"Unknown error occurred (errno: %d)", err);
@@ -121,6 +145,10 @@ ods_error_obexrsp2gerror (gint obex_response, GError **error)
 		return;
 		
 	switch (obex_response) {
+		case OBEX_RSP_REQUEST_TIME_OUT:
+			g_set_error (error, ODS_ERROR, ODS_ERROR_TIMEOUT,
+						"Request timeout");
+			break;
 		case OBEX_RSP_BAD_REQUEST:
 			g_set_error (error, ODS_ERROR, ODS_ERROR_BAD_REQUEST,
 						"Bad request");
@@ -145,6 +173,26 @@ ods_error_obexrsp2gerror (gint obex_response, GError **error)
 			g_set_error (error, ODS_ERROR, ODS_ERROR_SERVER_ERROR,
 						"Remote server error");
 			break;
+		case OBEX_RSP_NOT_MODIFIED:
+			g_set_error (error, ODS_ERROR, ODS_ERROR_NOT_MODIFIED,
+						"No modification done");
+			break;
+		case OBEX_RSP_CONFLICT:
+			g_set_error (error, ODS_ERROR, ODS_ERROR_CONFLICT,
+						"Conflict error");
+			break;
+		case OBEX_RSP_DATABASE_FULL:
+			g_set_error (error, ODS_ERROR, ODS_ERROR_DATABASE_ERROR,
+						"Could not write (database full)");
+			break;
+		case OBEX_RSP_DATABASE_LOCKED:
+			g_set_error (error, ODS_ERROR, ODS_ERROR_DATABASE_ERROR,
+						"Could not read/write (database locked)");
+			break;
+		case OBEX_RSP_UNSUPPORTED_MEDIA_TYPE:
+			g_set_error (error, ODS_ERROR, ODS_ERROR_UNSUPPORTED_MEDIA_TYPE,
+						"Media type not supported");
+			break;
 		default:
 			g_set_error (error, ODS_ERROR, ODS_ERROR_FAILED,
 						"Operation failed (unexpected response)");
