@@ -26,6 +26,7 @@
 #include <time.h>
 #include <errno.h>
 #include <string.h>
+#include <sys/statvfs.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -46,7 +47,9 @@
 #include "ods-error.h"
 #include "ods-server.h"
 #include "ods-session.h"
+#include "ods-marshal.h"
 #include "ods-manager.h"
+#include "ods-usb.h"
 #include "ods-manager-dbus-glue.h"
 
 #define ODS_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ODS_TYPE_MANAGER, OdsManagerPrivate))
@@ -54,23 +57,25 @@
 typedef struct OdsManagerSessionInfo_ {
 	OdsSession	*session;
 	/* Bluetooth specific */
-	gchar		*bluetooth_address;
+	OdsBluezCancellable	*bluetooth_cancel_data;
+	gchar				*bluetooth_target_address;
+	gchar				*bluetooth_source_address;
+	gint				bluetooth_channel;
+    /* USB specific */
+	gint				usb_interface_number;
 } OdsManagerSessionInfo;
 
 typedef struct OdsManagerServerInfo_ {
 	OdsServer	*server;
 	/* Bluetooth specific */
-	gchar		*bluetooth_address;
+	gchar		*bluetooth_source_address;
+	gboolean	require_pairing;
 	guint32		sdp_record_handle;
 } OdsManagerServerInfo;
 
 typedef struct OdsManagerCreateBluetoothSessionData_ {
-	/* arguments from function */
-	OdsManager				*manager;
-	gchar					*address;
-	gint					service;
-	/* DBUS data */
-	DBusGMethodInvocation	*context;
+	OdsManager	*manager;
+	OdsSession	*session;
 } OdsManagerCreateBluetoothSessionData;
 
 struct OdsManagerPrivate
@@ -82,15 +87,15 @@ struct OdsManagerPrivate
 	GHashTable	*session_list;
 	/* Server list (DBus path as key and OdsManagerServerInfo as value) */
 	GHashTable	*server_list;
-	OdsBluez	*bluez;
 	DBusGProxy	*dbus_proxy;
 	GHashTable	*listened_dbus_names;
 	GHashTable	*removed_dbus_names;
 };
 
 enum {
-	SESSION_CREATED,
-	SESSION_REMOVED,
+	SESSION_CONNECTED,
+	SESSION_CLOSED,
+	SESSION_CONNECT_ERROR,
 	DISPOSED,
 	LAST_SIGNAL
 };
@@ -155,7 +160,10 @@ ods_manager_listened_names_remove (OdsManager *manager, const gchar *dbus_owner,
 
 static void
 ods_manager_session_list_add (OdsManager *manager, OdsSession *session,
-								const gchar *bluetooth_address,
+								OdsBluezCancellable *bluetooth_cancel_data,
+								const gchar *bluetooth_target_address,
+								const gchar *bluetooth_source_address,
+								const gint usb_interface_number,
 								const gchar *dbus_owner,
 								const gchar *dbus_path)
 {
@@ -163,7 +171,10 @@ ods_manager_session_list_add (OdsManager *manager, OdsSession *session,
 	
 	session_info = g_new0 (OdsManagerSessionInfo, 1);
 	session_info->session = session;
-	session_info->bluetooth_address = g_strdup (bluetooth_address);
+	session_info->bluetooth_cancel_data = bluetooth_cancel_data;
+	session_info->bluetooth_target_address = g_strdup (bluetooth_target_address);
+	session_info->bluetooth_source_address = g_strdup (bluetooth_source_address);
+	session_info->usb_interface_number = usb_interface_number;
 	g_hash_table_insert (manager->priv->session_list, g_strdup (dbus_path),
 							session_info);
 	ods_manager_listened_names_add (manager, dbus_owner, dbus_path);
@@ -171,7 +182,8 @@ ods_manager_session_list_add (OdsManager *manager, OdsSession *session,
 
 static void
 ods_manager_server_list_add (OdsManager *manager, OdsServer *server,
-								const gchar *bluetooth_address,
+								const gchar *bluetooth_source_address,
+								gboolean require_pairing,
 								guint32 sdp_record_handle,
 								const gchar *dbus_owner,
 								const gchar *dbus_path)
@@ -180,7 +192,8 @@ ods_manager_server_list_add (OdsManager *manager, OdsServer *server,
 	
 	server_info = g_new0 (OdsManagerServerInfo, 1);
 	server_info->server = server;
-	server_info->bluetooth_address = g_strdup (bluetooth_address);
+	server_info->bluetooth_source_address = g_strdup (bluetooth_source_address);
+	server_info->require_pairing = require_pairing;
 	server_info->sdp_record_handle = sdp_record_handle;
 	g_hash_table_insert (manager->priv->server_list, g_strdup (dbus_path),
 							server_info);
@@ -190,14 +203,15 @@ ods_manager_server_list_add (OdsManager *manager, OdsServer *server,
 static void
 ods_manager_session_info_free (OdsManagerSessionInfo *session_info)
 {
-	g_free (session_info->bluetooth_address);
+	g_free (session_info->bluetooth_target_address);
+	g_free (session_info->bluetooth_source_address);
 	g_free (session_info);
 }
 
 static void
 ods_manager_server_info_free (OdsManagerServerInfo *server_info)
 {
-	g_free (server_info->bluetooth_address);
+	g_free (server_info->bluetooth_source_address);
 	g_free (server_info);
 }
 
@@ -243,12 +257,32 @@ session_closed_cb (OdsSession *session, OdsManager *manager)
 	g_message ("session closed");
 	g_object_get (session, "dbus-path", &session_object, NULL);
 	ods_manager_session_list_remove (manager, session);
-	g_signal_emit (manager, signals [SESSION_REMOVED], 0, session_object);
+	g_signal_emit (manager, signals [SESSION_CLOSED], 0, session_object);
 	
 	g_free (session_object);
 	g_object_unref (session);
 }
 
+static void
+session_connect_result_cb (OdsSession *session,	const gchar *error_name,
+							const gchar *error_message, OdsManager *manager)
+{
+	gchar	*session_object;
+	
+	g_message ("session_connect_result_cb");
+	g_object_get (session, "dbus-path", &session_object, NULL);
+	if (!error_name)
+		g_signal_emit (manager, signals [SESSION_CONNECTED], 0, session_object);
+	else {
+		g_signal_emit (manager, signals [SESSION_CONNECT_ERROR], 0, session_object,
+						error_name, error_message);
+		session_closed_cb (session, manager);
+	}
+	g_signal_handlers_disconnect_by_func (session,
+						G_CALLBACK (session_connect_result_cb), manager);
+	g_free (session_object);
+}
+
 static void
 server_disposed_cb (OdsServer *server, OdsManager *manager)
 {	
@@ -262,8 +296,9 @@ server_disposed_cb (OdsServer *server, OdsManager *manager)
 static void
 server_closed_cb (OdsServer *server, OdsManager *manager)
 {
+	g_message ("disposing server");
 	g_signal_connect (server, "disposed", G_CALLBACK (server_disposed_cb), manager);
-	g_object_run_dispose (G_OBJECT (server));
+	ods_server_dispose (server);
 }
 
 static void
@@ -271,20 +306,42 @@ server_started_cb (OdsServer *server, OdsManager *manager)
 {
 	OdsManagerServerInfo	*server_info;
 	gchar					*server_object;
+	gchar					*path;
 	guint					service;
+	ImagingSdpData			*imagingdata = NULL;
+	struct statvfs			sfs;
 	guint32					record_handle;
 	
 	/* Add service record to SDP database */
 	g_object_get (server, "dbus-path", &server_object, NULL);
+	g_object_get (server, "path", &path, NULL);
 	g_object_get (server, "service", &service, NULL);
 	server_info = g_hash_table_lookup (manager->priv->server_list, server_object);
 	
-	if (strlen (server_info->bluetooth_address) == 0)
+	if (strlen (server_info->bluetooth_source_address) == 0)
 		return;
 	
-	record_handle = ods_bluez_add_service_record (manager->priv->bluez,
-													server_info->bluetooth_address,
-													service);
+	if (service == ODS_SERVICE_BIP) {
+		/* ods currently supports ImagePush and RemoteDisplay
+		 * as Imaging Responder */
+		imagingdata = g_new0 (ImagingSdpData, 1);
+		imagingdata->supp_capabilities |= BIP_SUPP_CAP_GENERIC_IMAGING;
+		imagingdata->supp_capabilities |= BIP_SUPP_CAP_DISPLAYING;
+		imagingdata->supp_features |= BIP_SUPP_FEAT_IMAGE_PUSH;
+		imagingdata->supp_features |= BIP_SUPP_FEAT_REMOTE_DISPLAY;
+		imagingdata->supp_functions |= BIP_SUPP_FUNC_GET_CAPABILITIES;
+		imagingdata->supp_functions |= BIP_SUPP_FUNC_PUT_IMAGE;
+		imagingdata->supp_functions |= BIP_SUPP_FUNC_PUT_LINKED_ATTACHMENT;
+		imagingdata->supp_functions |= BIP_SUPP_FUNC_PUT_LINKED_THUMBNAIL;
+		imagingdata->supp_functions |= BIP_SUPP_FUNC_REMOTE_DISPLAY;
+		if (statvfs (path, &sfs) == -1)
+			imagingdata->data_capacity = 0;
+		else
+			imagingdata->data_capacity = (guint64)sfs.f_frsize * sfs.f_bfree;
+	}
+	record_handle = ods_bluez_add_service_record (
+										server_info->bluetooth_source_address,
+										service, imagingdata);
 	if (record_handle == 0) {
 		/* could not add SDP record */
 		g_warning ("Could not add SDP record for server (%s), closing server",
@@ -297,6 +354,9 @@ server_started_cb (OdsServer *server, OdsManager *manager)
 	}
 	
 	g_free (server_object);
+	g_free (path);
+	if (imagingdata)
+		g_free (imagingdata);
 }
 
 static void
@@ -313,9 +373,9 @@ server_stopped_cb (OdsServer *server, OdsManager *manager)
 	if (!server_info)
 		goto out;
 
-	if (server_info->sdp_record_handle && strlen (server_info->bluetooth_address) > 0) {
-		ods_bluez_remove_service_record (manager->priv->bluez,
-											server_info->bluetooth_address,
+	if (server_info->sdp_record_handle &&
+		strlen (server_info->bluetooth_source_address) > 0) {
+		ods_bluez_remove_service_record (server_info->bluetooth_source_address,
 											server_info->sdp_record_handle);
 	}
 	
@@ -382,86 +442,290 @@ dbus_name_owner_changed_cb (DBusGProxy *dbus_proxy, const gchar *name,
 }
 
 static void
-client_socket_connected_cb (gint fd, GError *error,
+client_socket_connected_cb (gint fd, gint rfcomm_channel,
+							const ImagingSdpData *imagingdata, GError *error,
 							OdsManagerCreateBluetoothSessionData *data)
 {
-	gboolean		name_exists = TRUE;
-	GError			*error_local = NULL;
-	OdsSession		*session;
-	gchar			*sender = NULL;
-	gchar			*session_object = NULL;
+	gchar					*session_object = NULL;
+	gchar					*owner = NULL;
+	OdsManagerSessionInfo	*session_info;
 	
+	g_object_get (data->session, "dbus-path", &session_object, NULL);
+	g_object_get (data->session, "owner", &owner, NULL);
+	
+	/* We need to reset cancel_data for this session so that it is 
+	 * properly finalized later on */
+	session_info = g_hash_table_lookup (data->manager->priv->session_list,
+										session_object);
+	g_assert (session_info);
+	if (session_info->bluetooth_cancel_data)
+		session_info->bluetooth_cancel_data = NULL;
+	session_info->bluetooth_channel = rfcomm_channel;
 	
 	if (fd == -1) {
-		/* Could not connect, return error */
-		dbus_g_method_return_error (data->context, error);
-		g_free (data->address);
-		g_free (data);
-		return;
+		gchar	*error_name;
+		
+		/* Could not connect, emit SessionConnectError */
+		error_name = ods_error_get_dbus_name (error);	
+		g_signal_emit (data->manager, signals [SESSION_CONNECT_ERROR], 0,
+						session_object, error_name, error->message);
+		g_free (error_name);
+		
+		/* Finalize session object*/
+		session_closed_cb (data->session, data->manager);
+		goto out;
 	}
-	sender = dbus_g_method_get_sender (data->context);
-	g_message ("Session created by: %s", sender);
-	/* It might be that sender disconnected from bus while connecting socket */
-	org_freedesktop_DBus_name_has_owner (data->manager->priv->dbus_proxy,
-											sender, &name_exists, &error_local);
-	if (error_local) {
-		/* ignore this */
-		g_clear_error (&error_local);
+	if (imagingdata) {
+		g_message ("Imaging SDP data:");
+		g_message (">> capabilities: %" G_GUINT16_FORMAT, imagingdata->supp_capabilities);
+		g_message (">> features: %" G_GUINT16_FORMAT, imagingdata->supp_features);
+		g_message (">> functions: %" G_GUINT32_FORMAT, imagingdata->supp_functions);
+		g_message (">> capacity: %" G_GUINT64_FORMAT, imagingdata->data_capacity);
+		
+		g_object_set (data->session, "imaging-sdp-data", imagingdata, NULL);
 	}
-	if (!name_exists) {
-		shutdown (fd, SHUT_RDWR);
-		close (fd);
-		g_message ("Session owner disconnected from bus, discarding session");
-		g_set_error (&error_local, ODS_ERROR, ODS_ERROR_FAILED,
-						"Session owner disconnected from bus");
-		dbus_g_method_return_error (data->context, error_local);
-		g_free (data->address);
-		g_free (data);
-		return;
-	}
-	/* create session object and return it's object path */
-	session = ods_session_new (fd, data->service, sender);
 	
-	/* add session to session list */
-	g_object_get (session, "dbus-path", &session_object, NULL);
-	ods_manager_session_list_add (data->manager, session, data->address,
-									sender, session_object);
+	g_message ("Session created by: %s", owner);
+	
+	/* Connect to the session objects signals. */
+	g_signal_connect (data->session, "closed",
+						G_CALLBACK (session_closed_cb), data->manager);
+	g_signal_connect (data->session, "connect-result-internal", 
+						G_CALLBACK (session_connect_result_cb), data->manager);
 
-	/* deal with signals */
-	g_signal_connect (session, "closed", 
-						G_CALLBACK (session_closed_cb), 
-						data->manager);
-	g_signal_emit (data->manager, signals [SESSION_CREATED], 0, session_object);
+	/* Set the connected socket as session fd.
+	 * The session will automatically do OBEX Connect */
+	g_object_set (data->session, "fd", fd, NULL);
 	
-	/* return session object path */
-	dbus_g_method_return (data->context, session_object);
-	
-	g_free (sender);
+out:
+	g_free (owner);
 	g_free (session_object);
-	g_free (data->address);
 	g_free (data);
 }
 
+static struct {
+	const char	*name;
+	uint16_t	class;
+	gint		service;
+} ods_services[] = {
+	{ "pbap",	PBAP_SVCLASS_ID,				ODS_SERVICE_PBAP},
+	{ "ftp",	OBEX_FILETRANS_SVCLASS_ID,		ODS_SERVICE_FTP	},
+	{ "bip",	IMAGING_RESPONDER_SVCLASS_ID,	ODS_SERVICE_BIP },
+	{ "opp",	OBEX_OBJPUSH_SVCLASS_ID,		ODS_SERVICE_OPP },
+	{ }
+};
+
 static gboolean
-bluetooth_init (OdsManager *manager)
+pattern2service (const char *pattern, uint16_t *uuid, gint *service)
 {
-	if (manager->priv->bluez == NULL)
-		manager->priv->bluez = ods_bluez_new ();
-	else if (!ods_bluez_is_initialized (manager->priv->bluez)) {
-		g_message ("Reinitializing OdsBluez object");
-		/* Try to initialize bluez again */
-		if (G_IS_OBJECT (manager->priv->bluez))
-			g_object_unref (manager->priv->bluez);
-		manager->priv->bluez = ods_bluez_new ();
+	int i;
+
+	for (i = 0; ods_services[i].name; i++) {
+		if (strcasecmp (ods_services[i].name, pattern) == 0) {
+			*uuid = ods_services[i].class;
+			*service = ods_services[i].service;
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static gboolean
+string2service (const char *string, uuid_t *uuid, gint *service)
+{
+	uint32_t data0, data4;
+	uint16_t data1, data2, data3, data5;
+
+	if (uuid && strlen(string) == 36 &&
+			string[8] == '-' &&
+			string[13] == '-' &&
+			string[18] == '-' &&
+			string[23] == '-' &&
+			sscanf(string, "%08x-%04hx-%04hx-%04hx-%08x%04hx",
+				&data0, &data1, &data2, &data3, &data4, &data5) == 6) {
+		uint8_t val[16];
+
+		data0 = g_htonl(data0);
+		data1 = g_htons(data1);
+		data2 = g_htons(data2);
+		data3 = g_htons(data3);
+		data4 = g_htonl(data4);
+		data5 = g_htons(data5);
+
+		memcpy(&val[0], &data0, 4);
+		memcpy(&val[4], &data1, 2);
+		memcpy(&val[6], &data2, 2);
+		memcpy(&val[8], &data3, 2);
+		memcpy(&val[10], &data4, 4);
+		memcpy(&val[14], &data5, 2);
+
+		sdp_uuid128_create (uuid, val);
+		/* we got UUID128, let's use generic GOEP service */
+		*service = ODS_SERVICE_GOEP;
+
+		return TRUE;
+	} else {
+		uint16_t class;
+		
+		/* got pattern specifying exactly which service to use */
+		if (pattern2service (string, &class, service)) {
+			if (uuid)
+				sdp_uuid16_create (uuid, class);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static gboolean
+create_bluetooth_session_full (OdsManager *manager,
+					const gchar *target_address,
+					const gchar *source_address,
+					const gchar *pattern,
+					const gchar *bip_feature,
+					DBusGMethodInvocation *context)
+{
+	GError		*error = NULL;
+	gboolean	ret = TRUE;
+	gboolean	valid;
+	bdaddr_t	src;
+	bdaddr_t	dst;
+	uuid_t		uuid;
+	gint		service;
+	gchar		**parsed = NULL;
+	OdsSession	*session;
+	gchar		*sender;
+	gchar		*session_object;
+	gchar		*target_uuid = NULL;
+	guint		imaging_feature = 0;
+	OdsBluezCancellable	*cancel_data;
+	
+	g_return_val_if_fail (manager != NULL, FALSE);
+	g_return_val_if_fail (ODS_IS_MANAGER (manager), FALSE);
+	
+	/* check target address validity */
+	str2ba (target_address, &dst);
+	/* can be only real address */
+	valid = bachk (target_address) == 0 &&
+			bacmp (&dst, BDADDR_ANY) &&
+			bacmp (&dst, BDADDR_ALL);
+	
+	if (!valid) {
+		g_set_error (&error, ODS_ERROR,	ODS_ERROR_INVALID_ARGUMENTS,
+						"Invalid target Bluetooth address");
+		dbus_g_method_return_error (context, error);
+		ret = FALSE;
+		goto out;
 	}
 	
-	return ods_bluez_is_initialized (manager->priv->bluez);
+	/* check address validity */
+	str2ba (source_address, &src);
+	/* can be real address or BDADDR_ANY */
+	valid = bachk (source_address) == 0 &&
+			bacmp (&src, BDADDR_ALL);
+		
+	if (!valid) {
+		g_set_error (&error, ODS_ERROR,	ODS_ERROR_INVALID_ARGUMENTS,
+						"Invalid source Bluetooth address");
+		dbus_g_method_return_error (context, error);
+		ret = FALSE;
+		goto out;
+	}
+	
+	/* format of pattern at this point should be <service/uuid128>[:<channel>] */
+	parsed = g_strsplit (pattern, ":", 2);
+	
+	/* convert pattern to service and bluetooth UUID */
+	if (!string2service (parsed[0], &uuid, &service)) {
+		g_set_error (&error, ODS_ERROR,	ODS_ERROR_INVALID_ARGUMENTS,
+						"Invalid pattern");
+		dbus_g_method_return_error (context, error);
+		ret = FALSE;
+		goto out;
+	}
+	
+	/* determine target UUID according to service */
+	switch (service) {
+		case ODS_SERVICE_FTP:
+			target_uuid = OBEX_FTP_UUID;
+			break;
+		case ODS_SERVICE_PBAP:
+			target_uuid = OBEX_PBAP_UUID;
+			break;
+		case ODS_SERVICE_BIP:
+			if (!g_ascii_strcasecmp (bip_feature, ODS_MANAGER_BIP_IMAGEPUSH_STR)) {
+				target_uuid = OBEX_BIP_IPUSH_UUID;
+				imaging_feature = BIP_SUPP_FEAT_IMAGE_PUSH;
+			} else if (!g_ascii_strcasecmp (bip_feature, ODS_MANAGER_BIP_REMOTEDISPLAY_STR)) {
+				target_uuid = OBEX_BIP_RD_UUID;
+				imaging_feature = BIP_SUPP_FEAT_REMOTE_DISPLAY;
+			} else {
+				g_set_error (&error, ODS_ERROR,	ODS_ERROR_INVALID_ARGUMENTS,
+								"Unknown imaging feature");
+				dbus_g_method_return_error (context, error);
+				ret = FALSE;
+				goto out;
+			}
+			break;
+	}
+	
+	/* Create a session object */
+	sender = dbus_g_method_get_sender (context);
+	session = ods_session_new (-1, -1, service, sender, target_uuid);
+	if (imaging_feature != 0)
+		g_object_set (session, "imaging-feature", imaging_feature, NULL);
+	g_object_get (session, "dbus-path", &session_object, NULL);
+	
+	/* connect Bluetooth transport (ods-bluez.c) */
+	OdsManagerCreateBluetoothSessionData *cb_data;/* data to pass to callback */
+	
+	cb_data = g_new0 (OdsManagerCreateBluetoothSessionData, 1);
+	cb_data->manager = manager;
+	cb_data->session = session;
+	
+	/* start connecting socket for session */
+	if (parsed[0])
+		g_message ("Parsed[0]: %s", parsed[0]);
+	if (parsed[1])
+		g_message ("Parsed[1]: %s", parsed[1]);
+	cancel_data = ods_bluez_get_client_socket (&dst, &src, &uuid,
+								imaging_feature,
+								(g_strv_length (parsed)>1 ? atoi (parsed[1]) : 0),
+								(OdsBluezFunc) client_socket_connected_cb,
+								cb_data);
+	ods_manager_session_list_add (cb_data->manager, cb_data->session,
+									cancel_data,
+									target_address, source_address, -1,
+									sender, session_object);
+	
+	if (!cancel_data) {
+		g_set_error (&error, ODS_ERROR, ODS_ERROR_CONNECTION_ATTEMPT_FAILED,
+						"Service search failed (%s)", strerror (errno));
+		client_socket_connected_cb (-1, -1, NULL, error, cb_data);
+		/* SessionConnectError will be emitted immediately after session obj is returned */
+	}
+	
+	/* Return session object now, SessionConnected will be emitted when
+	 * session is fully connected */
+	dbus_g_method_return (context, session_object);
+	g_free (sender);
+	g_free (session_object);
+	
+out:
+	if (error)
+		g_clear_error (&error);
+	if (parsed)
+		g_strfreev (parsed);
+	return ret;
 }
 
 /**
  * ods_manager_create_bluetooth_session:
  * @manager: This class instance
- * @address: Bluetooth address of remote device to connect to
+ * @target_address: Bluetooth address of remote device to connect to
+ * @source_address: Bluetooth address of local device (adapter)
  * @pattern: OBEX UUID (commonly "opp", "ftp", etc.)
  *
  * Creates and auto-connects Bluetooth session.
@@ -470,82 +734,219 @@ bluetooth_init (OdsManager *manager)
  **/
 gboolean
 ods_manager_create_bluetooth_session (OdsManager *manager,
-					const gchar *address,
+					const gchar *target_address,
+					const gchar *source_address,
+					const gchar *pattern,
+					DBusGMethodInvocation *context)
+{
+	return create_bluetooth_session_full (manager, target_address, source_address,
+											pattern, NULL, context);
+}
+
+/**
+ * ods_manager_create_usb_session:
+ * @manager: This class instance
+ * @interface_number: USB OBEX interface number to connect to
+ * @pattern: service used (commonly "ftp")
+ *
+ * Creates and auto-connects USB session.
+ *
+ * Return value: 
+ **/
+gboolean
+ods_manager_create_usb_session (OdsManager *manager,
+					const gint interface_number,
 					const gchar *pattern,
 					DBusGMethodInvocation *context)
 {
 	GError		*error = NULL;
-	gboolean	valid;
-	bdaddr_t	*bd_address;
+	gboolean	ret = TRUE;
+	OdsSession	*session;
 	gint		service;
-	gchar		**parsed;
-	
+	gchar		*target_uuid = NULL;
+	gchar		*sender;
+	gchar		*session_object;
 	
 	g_return_val_if_fail (manager != NULL, FALSE);
 	g_return_val_if_fail (ODS_IS_MANAGER (manager), FALSE);
 	
-	/* check address validity */
-	bd_address = strtoba (address);
-	/* can be only real address */
-	valid = bachk (address) == 0 &&
-			bacmp (bd_address, BDADDR_ANY) &&
-			bacmp (bd_address, BDADDR_ALL);
-	free (bd_address);
-	
-	if (!valid) {
+	if (interface_number < 0) {
 		g_set_error (&error, ODS_ERROR,	ODS_ERROR_INVALID_ARGUMENTS,
-						"Invalid Bluetooth address");
+						"Invalid USB interface number");
 		dbus_g_method_return_error (context, error);
-		g_clear_error (&error);			
-		return FALSE;
+		ret = FALSE;
+		goto out;
 	}
 	
-	/* check pattern validity (has to be supported/known service */
-	
-	parsed = g_strsplit (pattern, ":", 2);
-	
-	if (!g_ascii_strcasecmp (parsed[0], ODS_MANAGER_FTP_STR))
-		service = ODS_SERVICE_FTP;
-	else if (!g_ascii_strcasecmp (parsed[0], ODS_MANAGER_PBAP_STR))
-		service = ODS_SERVICE_PBAP;
-	else if (!g_ascii_strcasecmp (parsed[0], ODS_MANAGER_OPP_STR))
-		service = ODS_SERVICE_OPP;
-	else {
+	/* convert pattern to service and bluetooth UUID */
+	if (!string2service (pattern, NULL, &service)) {
 		g_set_error (&error, ODS_ERROR,	ODS_ERROR_INVALID_ARGUMENTS,
 						"Invalid pattern");
 		dbus_g_method_return_error (context, error);
-		g_clear_error (&error);
-		g_strfreev (parsed);
-		return FALSE;
+		ret = FALSE;
+		goto out;
 	}
 	
-	/* initialize Bluetooth */
-	if (!bluetooth_init (manager)) {
-		g_set_error (&error, ODS_ERROR,	ODS_ERROR_TRANSPORT_NOT_AVAILABLE,
-						"Bluez DBus interface not available");
-		dbus_g_method_return_error (context, error);
+	if (service == ODS_SERVICE_FTP)
+		target_uuid = OBEX_FTP_UUID;
+
+	/* Create a session object */
+	sender = dbus_g_method_get_sender (context);
+	session = ods_session_new (-1, -1, service, sender, target_uuid);
+	g_object_get (session, "dbus-path", &session_object, NULL);
+		
+	ods_manager_session_list_add (manager, session,
+									NULL, NULL, NULL, interface_number,
+									sender, session_object);
+	
+	g_message ("Session created by: %s", sender);
+	
+	/* Connect to the session objects signals. */
+	g_signal_connect (session, "closed",
+						G_CALLBACK (session_closed_cb), manager);
+	g_signal_connect (session, "connect-result-internal", 
+						G_CALLBACK (session_connect_result_cb), manager);
+	/* Set USB interface number.
+	 * The session will automatically do OBEX Connect */
+	g_object_set (session, "usbintfnum", interface_number, NULL);
+
+	/* Return session object now, SessionConnected will be emitted when
+	 * session is fully connected */
+	dbus_g_method_return (context, session_object);
+	g_free (sender);
+	g_free (session_object);
+	
+out:
+	if (error)
 		g_clear_error (&error);
-		g_strfreev (parsed);
-		return FALSE;
+	return ret;
+}
+
+/**
+ * ods_manager_get_usb_interfaces_num:
+ * @manager: This class instance
+ *
+ * Returns number of currently available USB OBEX interfaces
+ *
+ * Return value: 
+ **/
+guint
+ods_manager_get_usb_interfaces_num (OdsManager *manager)
+{
+#ifdef USE_USB
+	GList *list;
+	guint num_interfaces;
+
+	list = ods_usbobex_find_interfaces ();
+	num_interfaces = g_list_length (list);
+	ods_usbobex_free_interfaces (list);
+
+	return num_interfaces;
+#else
+	return 0;
+#endif /* USE_USB */
+}
+
+/**
+ * ods_manager_get_usb_interfaces_info:
+ * @manager: This class instance
+ * @interface_number: the number of interface to return information about
+ *
+ * Returns information about a specific USB USB OBEX interface
+ *
+ * Return value: 
+ **/
+GHashTable *
+ods_manager_get_usb_interface_info (OdsManager *manager, const gint interface_number)
+{
+#ifdef USE_USB
+	ods_usb_info *item;
+	GList *list;
+#endif
+	GHashTable *info;
+	
+	info = g_hash_table_new_full ((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal, NULL, (GDestroyNotify) g_free);
+
+#ifdef USE_USB
+	list = ods_usbobex_find_interfaces();
+	item = g_list_nth_data (list, interface_number);
+	if (item == NULL) {
+		g_warning("No such USB interface, requested interface %d info\n", interface_number);
+		return info;
 	}
-	/* connect Bluetooth transport (ods-bluez.c) */
-	OdsManagerCreateBluetoothSessionData *cb_data;/* data to pass to callback */
+
+	g_message ("item->path %s", item->path);
+
+	g_hash_table_insert (info, "Manufacturer", 
+								g_strdup (item->manufacturer));
+	g_hash_table_insert (info, "Product", 
+								g_strdup (item->product));
+	g_hash_table_insert (info, "Serial", 
+								g_strdup (item->serial));
+	g_hash_table_insert (info, "Configuration", 
+								g_strdup (item->configuration));
+	g_hash_table_insert (info, "ControlInterface", 
+								g_strdup (item->control_interface));
+	g_hash_table_insert (info, "DataInterfaceIdle", 
+								g_strdup (item->data_interface_idle));
+	g_hash_table_insert (info, "DataInterfaceActive", 
+								g_strdup (item->data_interface_active));
+	g_hash_table_insert (info, "Path", 
+								g_strdup (item->path));
+
+	ods_usbobex_free_interfaces (list);
+#endif /* USE_USB */
+
+	return info;
+}
+
+gboolean
+ods_manager_create_bluetooth_imaging_session (OdsManager *manager,
+					const gchar *target_address,
+					const gchar *source_address,
+					const gchar *bip_feature,
+					DBusGMethodInvocation *context)
+{
+	gchar		**parsed;
+	gchar		*pattern;
+	gboolean	ret;
 	
-	cb_data = g_new0 (OdsManagerCreateBluetoothSessionData, 1);
-	cb_data->manager = manager;
-	cb_data->address = g_strdup (address);
-	cb_data->service = service;
-	cb_data->context = context;
+	parsed = g_strsplit (bip_feature, ":", 2);
+	if (g_strv_length (parsed)>1)
+		pattern = g_strdup_printf (ODS_MANAGER_BIP_STR ":%s", parsed[1]);
+	else
+		pattern = g_strdup (ODS_MANAGER_BIP_STR);
 	
-	g_message ("Parsed[0]: %s, Parsed[1]: %s", parsed[0], parsed[1]);
-	ods_bluez_get_client_socket (manager->priv->bluez, address, parsed[0],
-									(g_strv_length (parsed)>1 ? atoi (parsed[1]) : 0),
-									(OdsBluezFunc) client_socket_connected_cb,
-									cb_data);
+	ret = create_bluetooth_session_full (manager, target_address, source_address,
+											pattern, parsed[0], context);
+		
 	g_strfreev (parsed);
+	g_free (pattern);
+	return ret;
+}
+
+static gboolean
+cancel_session_connect (OdsManager *manager,
+						OdsManagerSessionInfo *session_info)
+{
+	if (session_info->bluetooth_cancel_data) {
+		ods_bluez_cancel_get_client_socket (session_info->bluetooth_cancel_data);
+		return TRUE;
+	}
+	/* for future reference: Check for other transports cancellables */
+	return FALSE;
+}
+
+gboolean
+ods_manager_cancel_session_connect (OdsManager *manager,
+										const gchar *session_object)
+{
+	OdsManagerSessionInfo	*session_info;
 	
-	/* Session object will be created in client_socket_connected_cb */
-	return TRUE;
+	session_info = g_hash_table_lookup (manager->priv->session_list, session_object);
+	if (!session_info)
+		return FALSE;
+	return cancel_session_connect (manager, session_info);
 }
 
 gboolean
@@ -557,7 +958,7 @@ ods_manager_create_bluetooth_server (OdsManager *manager,
 {
 	GError		*error = NULL;
 	gboolean	valid;
-	bdaddr_t	*bd_address;
+	bdaddr_t	bd_address;
 	gint		service;
 	guint8		channel;
 	gint		fd;
@@ -570,11 +971,10 @@ ods_manager_create_bluetooth_server (OdsManager *manager,
 	g_return_val_if_fail (ODS_IS_MANAGER (manager), FALSE);
 	
 	/* check address validity */
-	bd_address = strtoba (source_address);
+	str2ba (source_address, &bd_address);
 	/* can be real address or BDADDR_ANY */
 	valid = bachk (source_address) == 0 &&
-			bacmp (bd_address, BDADDR_ALL);
-	free (bd_address);
+			bacmp (&bd_address, BDADDR_ALL);
 	
 	if (!valid) {
 		g_set_error (&error, ODS_ERROR,	ODS_ERROR_INVALID_ARGUMENTS,
@@ -594,6 +994,9 @@ ods_manager_create_bluetooth_server (OdsManager *manager,
 	} else if (!g_ascii_strcasecmp (pattern, ODS_MANAGER_PBAP_STR)) {
 		service = ODS_SERVICE_PBAP;
 		channel = ODS_PBAP_RFCOMM_CHANNEL;
+	} else if (!g_ascii_strcasecmp (pattern, ODS_MANAGER_BIP_STR)) {
+		service = ODS_SERVICE_BIP;
+		channel = ODS_BIP_RFCOMM_CHANNEL;
 	} else {
 		g_set_error (&error, ODS_ERROR,	ODS_ERROR_INVALID_ARGUMENTS,
 						"Invalid pattern");
@@ -602,17 +1005,8 @@ ods_manager_create_bluetooth_server (OdsManager *manager,
 		return FALSE;
 	}
 	
-	/* initialize Bluetooth */
-	if (!bluetooth_init (manager)) {
-		g_set_error (&error, ODS_ERROR,	ODS_ERROR_TRANSPORT_NOT_AVAILABLE,
-						"Bluez DBus interface not available");
-		dbus_g_method_return_error (context, error);
-		g_clear_error (&error);
-		return FALSE;
-	}
 	/* create server socket */
-	fd = ods_bluez_get_server_socket (manager->priv->bluez, source_address,
-										channel);
+	fd = ods_bluez_get_server_socket (source_address, channel);
 	if (fd == -1) {
 		/* could not create server socket */
 		g_set_error (&error, ODS_ERROR, ODS_ERROR_FAILED,
@@ -625,7 +1019,7 @@ ods_manager_create_bluetooth_server (OdsManager *manager,
 	
 	/* require_pairing */
 	if (require_pairing) {
-		sockopt = RFCOMM_LM_SECURE;
+		sockopt = RFCOMM_LM_AUTH | RFCOMM_LM_ENCRYPT;
 		if (setsockopt (fd, SOL_RFCOMM, RFCOMM_LM, &sockopt, sizeof (sockopt)) < 0) {
 			g_set_error (&error, ODS_ERROR, ODS_ERROR_FAILED,
 							"Setting RFCOMM link mode failed");
@@ -642,8 +1036,8 @@ ods_manager_create_bluetooth_server (OdsManager *manager,
 	
 	/* add server to server list */
 	g_object_get (server, "dbus-path", &server_object, NULL);
-	ods_manager_server_list_add (manager, server, source_address, 0, sender,
-									server_object);
+	ods_manager_server_list_add (manager, server, source_address, require_pairing,
+									0, sender, server_object);
 	
 	/* deal with signals */
 	g_signal_connect (server, "started", G_CALLBACK (server_started_cb), manager);
@@ -667,8 +1061,12 @@ ods_manager_get_session_info (OdsManager *manager, gchar *session_object)
 	info = g_hash_table_new ((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal);
 	session_info = g_hash_table_lookup (manager->priv->session_list, session_object);
 	if (session_info) {
-		g_hash_table_insert (info, "BluetoothAddress", 
-								g_strdup (session_info->bluetooth_address));
+		g_hash_table_insert (info, "BluetoothTargetAddress", 
+								g_strdup (session_info->bluetooth_target_address));
+		g_hash_table_insert (info, "BluetoothSourceAddress", 
+								g_strdup (session_info->bluetooth_source_address));
+		g_hash_table_insert (info, "BluetoothChannel",
+								g_strdup_printf ("%d", session_info->bluetooth_channel));
 	}
 	return info;
 }
@@ -682,12 +1080,32 @@ ods_manager_get_server_info (OdsManager *manager, gchar *server_object)
 	info = g_hash_table_new ((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal);
 	server_info = g_hash_table_lookup (manager->priv->server_list, server_object);
 	if (server_info) {
-		g_hash_table_insert (info, "BluetoothAddress", 
-								g_strdup (server_info->bluetooth_address));
+		g_hash_table_insert (info, "BluetoothSourceAddress", 
+								g_strdup (server_info->bluetooth_source_address));
+		g_hash_table_insert (info, "RequirePairing",
+								g_strdup (server_info->require_pairing ? "True": "False"));
 	}
 	return info;
 }
 
+gchar**
+ods_manager_get_session_list (OdsManager *manager)
+{
+	return ods_hash_table_keys2strv (manager->priv->session_list);
+}
+
+gchar**
+ods_manager_get_server_list (OdsManager *manager)
+{
+	return ods_hash_table_keys2strv (manager->priv->server_list);
+}
+
+gchar*
+ods_manager_get_version (OdsManager *manager)
+{
+	return g_strdup_printf ("%s:%d", PACKAGE_VERSION, ODS_API_VERSION);
+}
+
 /**
  * ods_manager_class_init:
  * @klass: The OdsManagerClass
@@ -699,26 +1117,36 @@ ods_manager_class_init (OdsManagerClass *klass)
 
 	object_class->finalize = ods_manager_finalize;
 
-	signals [SESSION_CREATED] =
-		g_signal_new ("session-created",
+	signals [SESSION_CONNECTED] =
+		g_signal_new ("session-connected",
 			      G_TYPE_FROM_CLASS (object_class), 
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (OdsManagerClass, session_created),
+			      G_STRUCT_OFFSET (OdsManagerClass, session_connected),
 			      NULL, 
 			      NULL, 
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1, DBUS_TYPE_G_OBJECT_PATH);
 			      
-	signals [SESSION_REMOVED] =
-		g_signal_new ("session-removed",
+	signals [SESSION_CLOSED] =
+		g_signal_new ("session-closed",
 			      G_TYPE_FROM_CLASS (object_class), 
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (OdsManagerClass, session_removed),
+			      G_STRUCT_OFFSET (OdsManagerClass, session_closed),
 			      NULL, 
 			      NULL, 
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1, DBUS_TYPE_G_OBJECT_PATH);
 	
+	signals [SESSION_CONNECT_ERROR] =
+		g_signal_new ("session-connect-error",
+			      G_TYPE_FROM_CLASS (object_class), 
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (OdsManagerClass, session_connect_error),
+			      NULL, 
+			      NULL,
+			      ods_marshal_VOID__STRING_STRING_STRING,
+			      G_TYPE_NONE, 3, DBUS_TYPE_G_OBJECT_PATH, G_TYPE_STRING, G_TYPE_STRING);
+	
 	signals [DISPOSED] =
 		g_signal_new ("disposed",
 			      G_TYPE_FROM_CLASS (object_class), 
@@ -792,6 +1220,11 @@ ods_manager_session_finalize (gpointer key, OdsManagerSessionInfo *session_info,
 								OdsManager *manager)
 {
 	g_message ("attempting to close session");
+	
+	/* If session is being connected, cancel */
+	if (cancel_session_connect (manager, session_info))
+		return TRUE;
+	/* Finalize already connected session */
 	g_signal_connect (session_info->session, "cancelled", 
 						G_CALLBACK (session_cancelled_cb), manager);
 	ods_session_cancel_internal (session_info->session);
@@ -862,11 +1295,10 @@ ods_manager_finalize (GObject *object)
 	g_return_if_fail (manager->priv->disposed);
 	
 	g_message ("Finalizing manager");
+	ods_bluez_finalize();
 	g_hash_table_unref (manager->priv->listened_dbus_names);
 	g_hash_table_unref (manager->priv->session_list);
 	g_hash_table_unref (manager->priv->server_list);
-	if (G_IS_OBJECT (manager->priv->bluez))
-		g_object_unref (manager->priv->bluez);
 	g_object_unref (G_OBJECT (manager->priv->dbus_proxy));
 
 	G_OBJECT_CLASS (ods_manager_parent_class)->finalize (object);
