@@ -34,6 +34,10 @@ G_BEGIN_DECLS
 #define ODS_MANAGER_OPP_STR		"opp"
 #define ODS_MANAGER_FTP_STR		"ftp"
 #define ODS_MANAGER_PBAP_STR	"pbap"
+#define ODS_MANAGER_BIP_STR		"bip"
+
+#define ODS_MANAGER_BIP_IMAGEPUSH_STR		"imagepush"
+#define ODS_MANAGER_BIP_REMOTEDISPLAY_STR	"remotedisplay"
 
 #define ODS_TYPE_MANAGER	 (ods_manager_get_type ())
 #define ODS_MANAGER(o)		 (G_TYPE_CHECK_INSTANCE_CAST ((o), ODS_TYPE_MANAGER, OdsManager))
@@ -53,10 +57,14 @@ typedef struct
 typedef struct
 {
 	GObjectClass	parent_class;
-	void		(* session_created)		(OdsManager	*manager,
+	void		(* session_connected)	(OdsManager	*manager,
 							 			const gchar *session_object);
-	void		(* session_removed)		(OdsManager	*manager,
+	void		(* session_closed)		(OdsManager	*manager,
 							 			const gchar *session_object);
+	void		(* session_connect_error)(OdsManager *manager,
+											const gchar *session_object,
+											const gchar *error_name,
+											const gchar *error_message);
 	void		(* disposed)			(OdsManager	*manager);
 	DBusGConnection *connection;
 } OdsManagerClass;
@@ -67,9 +75,24 @@ OdsManager	*ods_manager_new				(void);
 gboolean	 ods_manager_is_initialized		(OdsManager *manager);
 
 gboolean	 ods_manager_create_bluetooth_session (OdsManager *manager,
-											const gchar *address,
+											const gchar *target_address,
+											const gchar *source_address,
 											const gchar *pattern,
 											DBusGMethodInvocation *context);
+gboolean	 ods_manager_create_bluetooth_imaging_session (OdsManager *manager,
+											const gchar *target_address,
+											const gchar *source_address,
+											const gchar *bip_feature,
+											DBusGMethodInvocation *context);
+gboolean	 ods_manager_create_usb_session (OdsManager *manager,
+											const gint interface_number,
+											const gchar *pattern,
+											DBusGMethodInvocation *context);
+guint		 ods_manager_get_usb_interfaces_num (OdsManager *manager);
+GHashTable	*ods_manager_get_usb_interface_info (OdsManager *manager,
+											const gint interface_number);
+gboolean	 ods_manager_cancel_session_connect (OdsManager *manager,
+											const gchar *session_object);
 gboolean	 ods_manager_create_bluetooth_server (OdsManager *manager,
 											const gchar *source_address,
 											const gchar *pattern,
@@ -78,6 +101,9 @@ gboolean	 ods_manager_create_bluetooth_server (OdsManager *manager,
 void		 ods_manager_dispose			(OdsManager	*manager);
 GHashTable	*ods_manager_get_session_info (OdsManager *manager, gchar *session_object);
 GHashTable	*ods_manager_get_server_info (OdsManager *manager, gchar *server_object);
+gchar		**ods_manager_get_session_list (OdsManager *manager);
+gchar		**ods_manager_get_server_list (OdsManager *manager);
+gchar		*ods_manager_get_version (OdsManager *manager);
 
 G_END_DECLS
 
