@@ -51,14 +51,22 @@
 
 static void     ods_server_session_class_init	(OdsServerSessionClass *klass);
 static void     ods_server_session_init			(OdsServerSession *server_session);
-static GObject* ods_server_session_constructor	(GType type, guint n_construct_params, 
-										GObjectConstructParam *construct_params);
 static void     ods_server_session_finalize		(GObject		*object);
 
 #define ODS_SERVER_SESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ODS_TYPE_SERVER_SESSION, OdsServerSessionPrivate))
 
-#define ODS_SERVER_SESSION_LOCK(server_session) g_message ("LOCK"); g_static_mutex_lock (&(server_session)->priv->mutex)
-#define ODS_SERVER_SESSION_UNLOCK(server_session) g_message ("UNLOCK"); g_static_mutex_unlock (&(server_session)->priv->mutex)
+#define ODS_SERVER_SESSION_LOCK(server_session) g_message ("LOCK %s", __FUNCTION__); g_static_mutex_lock (&(server_session)->priv->mutex)
+#define ODS_SERVER_SESSION_UNLOCK(server_session) g_message ("UNLOCK %s", __FUNCTION__); g_static_mutex_unlock (&(server_session)->priv->mutex)
+
+#define	ODS_SERVER_SESSION_DBUS_PATH_PATTERN	"/org/openobex/serversession%d"
+#define	ODS_SERVER_SESSION_DBUS_INTERFACE		"org.openobex.ServerSession"
+
+typedef enum {
+	ODS_SERVER_SESSION_STATE_NOT_CONNECTED,
+	ODS_SERVER_SESSION_STATE_OPEN,
+	ODS_SERVER_SESSION_STATE_BUSY
+} OdsServerSessionState;
+
 
 struct OdsServerSessionPrivate
 {
@@ -70,6 +78,7 @@ struct OdsServerSessionPrivate
 	gchar					*owner; /* D-Bus client, who initiated this session */
 	gboolean				allow_write; /* Whether to allow changes in file system */
 	gboolean				auto_accept; /* Whether incoming files should be auto-accepted */
+	gboolean				require_imaging_thumbnails;/* whether to request thumbnail after PutImage is completed */
 	/* state (open or busy) */
 	OdsServerSessionState	state; /* ODS_SERVER_SESSION_STATE_OPEN by default */
 	/* OBEX connection */
@@ -79,6 +88,9 @@ struct OdsServerSessionPrivate
 	GStaticMutex			mutex;
 	DBusGMethodInvocation	*dbus_context; /* D-Bus context for async methods */
 	gchar					*dbus_path; /* D-Bus path for this object */
+	/* BIP-specific */
+	GHashTable				*img_handle_list;/* hash list associating image handles with real filenames */
+	guint					selected_img;/* selected image handle (for RemoteDisplay functions) */
 };
 
 enum {
@@ -88,6 +100,7 @@ enum {
 	TRANSFER_PROGRESS,
 	TRANSFER_COMPLETED,
 	ERROR_OCCURRED,
+	REMOTE_DISPLAY_REQUESTED,
 	LAST_SIGNAL
 };
 
@@ -102,10 +115,82 @@ G_DEFINE_TYPE (OdsServerSession, ods_server_session, G_TYPE_OBJECT)
 static gboolean
 emit_disconnected (OdsServerSession *server_session)
 {
+	g_source_remove (server_session->priv->io_watch);
 	g_signal_emit (server_session, signals [DISCONNECTED], 0);
 	return FALSE;
 }
 
+/* adds uploaded image to list and returns it's image handle */
+static const gchar*
+image_list_add (OdsServerSession *server_session, const gchar *filename)
+{
+	gchar *img_handle;
+	
+	if (!server_session->priv->img_handle_list) {
+		server_session->priv->img_handle_list = g_hash_table_new_full (
+													g_str_hash, g_str_equal,
+													g_free, g_free);
+	}
+	img_handle = g_strdup_printf ("%07u",
+					g_hash_table_size (server_session->priv->img_handle_list)+1);
+	g_hash_table_insert (server_session->priv->img_handle_list, img_handle,
+							g_strdup (filename));
+	return img_handle;
+}
+
+static void
+remote_display_action (OdsServerSession *server_session, guint8 action)
+{
+	OdsServerSessionPrivate	*priv;
+	gchar					*img_handle = NULL;
+	gchar					*filename;
+	
+	priv = server_session->priv;
+	if (action == 0)
+		return;
+	
+	switch (action) {
+		case BIP_REMOTEDISPLAY_NEXTIMAGE:
+			if (priv->selected_img < g_hash_table_size (priv->img_handle_list))
+				priv->selected_img++;
+			break;
+		case BIP_REMOTEDISPLAY_PREVIOUSIMAGE:
+			if (priv->selected_img > 1)
+				priv->selected_img--;
+			break;
+		case BIP_REMOTEDISPLAY_SELECTIMAGE:
+			priv->selected_img = atoi (priv->obex_context->img_handle);
+			if (priv->selected_img > g_hash_table_size (priv->img_handle_list) ||
+					priv->selected_img < 1) {
+				/* resort to first */
+				priv->selected_img = 1;
+			} else {
+				/* valid value, use this img_handle */
+				img_handle = g_strdup (priv->obex_context->img_handle);
+			}
+			break;
+		case BIP_REMOTEDISPLAY_CURRENTIMAGE:
+			break;
+		default:
+			g_message ("Invalid RemoteDisplay action specified");
+			return;
+	}
+	if (!img_handle) {
+		/* get img_handle from selected_img */
+		img_handle = g_strdup_printf ("%07u", priv->selected_img);
+	}
+	/* lookup filename for selected img_handle */
+	if (!(filename = g_hash_table_lookup (priv->img_handle_list, img_handle))) {
+		/* should never happen since we did all bounds checking */
+		g_warning ("Selected img_handle not found in img_handle_list");
+		return;
+	}
+	/* finally emit RemoteDisplayRequested */
+	g_signal_emit (server_session, signals [REMOTE_DISPLAY_REQUESTED], 0,
+						filename);
+	g_free (img_handle);
+}
+
 static gboolean
 obex_io_callback (GIOChannel *io_channel, GIOCondition cond, gpointer data)
 {
@@ -166,13 +251,171 @@ obex_transfer_data_exchange_done (OdsServerSession *server_session, gint ret)
 		g_clear_error (&error);
 		/* Reset state */
 		server_session->priv->state = ODS_SERVER_SESSION_STATE_OPEN;
-	} else if (obex_context->report_progress &&
-				!obex_context->transfer_started_signal_emitted) {
+	} else if (ret == 0 && !obex_context->transfer_started_signal_emitted &&
+				obex_context->report_progress && obex_context->local) {
 		g_signal_emit (server_session, signals [TRANSFER_STARTED], 0,
 						obex_context->remote,
 						obex_context->local,
 						obex_context->target_size);
 		obex_context->transfer_started_signal_emitted = TRUE;
+		g_message ("TransferStarted emitted at obex_transfer_data_exchange_done");
+	}
+}
+
+static void
+send_image_handle (OdsServerSession *server_session, OdsObexContext *obex_context,
+					obex_object_t *object)
+{
+	/* In the end of BIP PutImage request ImageHandle has to be sent
+	 * Make sure that we have transfer info (PUT request was received).
+	 * This might be called on last readstream but REQCHECK event
+	 * might not have happened yet (happens with small files). */
+	const gchar			*img_handle;
+	gchar				*uhandle = NULL;
+	gsize				uhandle_len = 0;
+	obex_headerdata_t	hv;
+	gint				ret;
+	
+	if (server_session->priv->service == ODS_SERVICE_BIP &&
+			obex_context->obex_cmd == OBEX_CMD_PUT && obex_context->local &&
+			!g_ascii_strncasecmp (obex_context->type, BIP_IMG_TYPE,
+			strlen (obex_context->type))) {
+		
+		img_handle = image_list_add (server_session, obex_context->local);
+		uhandle_len = ods_filename_to_utf16 (&uhandle, img_handle);
+		if (uhandle == NULL) {
+			g_warning ("Failed to convert img_handle");
+			return;
+		}
+		hv.bs = (guchar *)uhandle;
+		ret = OBEX_ObjectAddHeader (obex_context->obex_handle, object,
+									OBEX_HDR_IMG_HANDLE, hv,
+									uhandle_len, 0);
+		g_free (uhandle);
+		if (ret < 0) {
+			g_warning ("Failed to add header");
+			return;
+		}
+		if (server_session->priv->require_imaging_thumbnails)
+			OBEX_ObjectSetRsp (object, OBEX_RSP_PARTIAL_CONTENT, OBEX_RSP_PARTIAL_CONTENT);
+		else
+			OBEX_ObjectSetRsp (object, OBEX_RSP_SUCCESS, OBEX_RSP_SUCCESS);
+	}
+}
+
+static void
+obex_request_cancelled (OdsServerSession *server_session, OdsObexContext *obex_context)
+{
+	/* Cleanup transfer stuff and reset state */
+	/* If it was PUT operation, remove incomplete file */
+	if (obex_context->obex_cmd == OBEX_CMD_PUT &&
+			obex_context->stream_fd >= 0)
+		g_unlink (obex_context->local);
+	ods_obex_transfer_close (obex_context);
+	server_session->priv->state = ODS_SERVER_SESSION_STATE_OPEN;
+	/* emit CANCELLED signal */
+	g_signal_emit (server_session, signals [CANCELLED], 0);
+	
+	/* In case this was trigerred by Cancel method */
+	if (server_session->priv->dbus_context) {
+		dbus_g_method_return (server_session->priv->dbus_context);
+		server_session->priv->dbus_context = NULL;
+		ODS_SERVER_SESSION_UNLOCK (server_session);
+	}
+}
+
+static gint
+obex_suspend_request (OdsServerSession *server_session, OdsObexContext *obex_context)
+{
+	/* only suspend if TransferStarted signal was already emitted */
+	if (!server_session->priv->auto_accept &&
+			obex_context->transfer_started_signal_emitted &&
+			!obex_context->suspend_result) {
+		ods_obex_transfer_suspend (obex_context);
+		
+		if (obex_context->suspend_result == OBEX_SUSP_REJECTED)
+			ods_server_session_cancel_internal (server_session);
+		return obex_context->suspend_result;
+	}
+	return 0;
+}
+
+static void
+obex_request_put (OdsServerSession *server_session, OdsObexContext *obex_context,
+					obex_object_t *object)
+{
+	guint8		action = 0;/* only used for BIP RemoteDisplay*/
+	gint		ret, suspend_ret;
+	gboolean	no_response_on_success = FALSE;
+	gchar		*img_filename;
+	
+	/* Check if we already have all transfer info
+	* because both EV_REQCHECK and EV_REQ trigger this function */
+	if (obex_context->stream_fd >= 0)
+		return;
+	
+	if (!server_session->priv->allow_write) {
+		g_message ("CMD_PUT forbidden");
+		OBEX_ObjectSetRsp (object, OBEX_RSP_FORBIDDEN, 
+							OBEX_RSP_FORBIDDEN);
+		return;
+	}
+	/* Check if we have write permissions for current path */
+	if (g_access (server_session->priv->path, W_OK) < 0) {
+		g_message ("No write permissions for current path");
+		OBEX_ObjectSetRsp (object, OBEX_RSP_UNAUTHORIZED, 
+							OBEX_RSP_UNAUTHORIZED);
+		return;
+	}
+	/* If auto_accept is False and transfer hasn't been suspended yet,
+	 * we will not send response until the transfer is accepted or rejected */
+	if (!server_session->priv->auto_accept && !obex_context->suspend_result)
+		no_response_on_success = TRUE;
+	ret = ods_obex_srv_put (obex_context, object, 
+					server_session->priv->path, &action, no_response_on_success);
+	g_message ("ret=%d", ret);
+	/* Add ImageFilename to ExtInfo according to ImageHandle (for BIP) */
+	if (ret == 0 && obex_context->img_handle) {
+		img_filename = g_hash_table_lookup (server_session->priv->img_handle_list,
+											obex_context->img_handle);
+		if (!img_filename)
+			g_message ("Invalid ImgHandle received");
+		else
+			ods_obex_transfer_add_info (obex_context, "ImageFilename",
+										g_strdup (img_filename));
+	}
+	/* Process BIP RemoteDisplay info (if available) */
+	if (ret == 0)
+		remote_display_action (server_session, action);
+	/* also emit TransferStarted signal */
+	if (ret == 0 && !obex_context->transfer_started_signal_emitted &&
+			obex_context->report_progress && obex_context->local) {
+		g_signal_emit (server_session, signals [TRANSFER_STARTED],
+						0, obex_context->remote,
+						obex_context->local,
+						obex_context->target_size);
+		obex_context->transfer_started_signal_emitted = TRUE;
+		g_message ("TransferStarted emitted at obex_request_put");
+	}
+	/* In a rare situation when received file is smaller than MTU,
+	 * this function is called after all data is already received.
+	 * We have to suspend request if auto_accept is False;
+	 * For BIP, image handle has to be sent as well */
+	if (ret == 0) {
+		suspend_ret = obex_suspend_request (server_session, obex_context);
+		if (suspend_ret == OBEX_SUSP_REJECTED) {
+			/* Transfer was rejected and no more data will be received so we
+			 * delete written file, emit Cancelled signal and send RSP_FORBIDDEN */
+			obex_request_cancelled (server_session, obex_context);
+			OBEX_ObjectSetRsp (object, OBEX_RSP_FORBIDDEN, OBEX_RSP_FORBIDDEN);
+		} else if (suspend_ret == OBEX_SUSP_ACCEPTED) {
+			/* Transfer was accepted, send RSP_SUCCESS */
+			OBEX_ObjectSetRsp (object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
+		}
+		if (suspend_ret != OBEX_SUSP_REJECTED) {
+			/* Transfer was accepted either now or earlier */
+			send_image_handle (server_session, obex_context, object);
+		}
 	}
 }
 
@@ -202,23 +445,7 @@ obex_event (obex_t *handle, obex_object_t *object, int mode, int event,
 			 * When link error really happens, it is handled in io_callback */
 			g_warning ("EV_LINKERR");
 		case OBEX_EV_ABORT:
-			/* Cleanup transfer stuff and reset state */
-			/* If it was PUT operation, remove incomplete file */
-			if (obex_context->obex_cmd == OBEX_CMD_PUT &&
-					obex_context->stream_fd >= 0)
-				g_unlink (obex_context->local);
-			ods_obex_transfer_close (obex_context);
-			server_session->priv->state = ODS_SERVER_SESSION_STATE_OPEN;
-			/* emit CANCELLED signal */
-			g_signal_emit (server_session, signals [CANCELLED], 0);
-			OBEX_ObjectSetRsp (object, OBEX_RSP_SUCCESS, OBEX_RSP_SUCCESS);
-			
-			/* In case this was trigerred by Cancel method */
-			if (server_session->priv->dbus_context) {
-				dbus_g_method_return (server_session->priv->dbus_context);
-				server_session->priv->dbus_context = NULL;
-				ODS_SERVER_SESSION_UNLOCK (server_session);
-			}
+			obex_request_cancelled (server_session, obex_context);
 			break;
 		case OBEX_EV_REQDONE:
 			switch (command) {
@@ -250,7 +477,6 @@ obex_event (obex_t *handle, obex_object_t *object, int mode, int event,
 					obex_context->obex_cmd = OBEX_CMD_PUT;
 					/* Initialize transfer and set state */
 					ods_obex_transfer_new (obex_context, NULL, NULL, NULL);
-					obex_context->report_progress = FALSE;/* might be delete operation */
 					server_session->priv->state = ODS_SERVER_SESSION_STATE_BUSY;
 					
 					OBEX_ObjectSetRsp (object, OBEX_RSP_CONTINUE, 
@@ -280,30 +506,7 @@ obex_event (obex_t *handle, obex_object_t *object, int mode, int event,
 		case OBEX_EV_REQCHECK:
 			if (command == OBEX_CMD_PUT) {
 				g_message ("CMD_PUT requested at REQCHECK");
-				if (!server_session->priv->allow_write) {
-					g_message ("CMD_PUT forbidden");
-					OBEX_ObjectSetRsp (object, OBEX_RSP_FORBIDDEN, 
-										OBEX_RSP_FORBIDDEN);
-					return;
-				}
-				/* Check if we have write permissions for current path */
-				if (g_access (server_session->priv->path, W_OK) < 0) {
-					g_message ("No write permissions for current path");
-					OBEX_ObjectSetRsp (object, OBEX_RSP_UNAUTHORIZED, 
-										OBEX_RSP_UNAUTHORIZED);
-					return;
-				}
-				ret = ods_obex_srv_put (obex_context, object, 
-										server_session->priv->path);
-				g_message ("ret=%d", ret);
-				/* also emit TransferStarted signal */
-				if (ret == 0 && obex_context->report_progress) {
-					g_signal_emit (server_session, signals [TRANSFER_STARTED],
-									0, obex_context->remote,
-									obex_context->local,
-									obex_context->target_size);
-					obex_context->transfer_started_signal_emitted = TRUE;
-				}
+				obex_request_put (server_session, obex_context, object);
 			}
 			break;
 		case OBEX_EV_REQ:
@@ -348,33 +551,12 @@ obex_event (obex_t *handle, obex_object_t *object, int mode, int event,
 										obex_context->local,
 										obex_context->target_size);
 						obex_context->transfer_started_signal_emitted = TRUE;
+						g_message ("TransferStarted emitted at obex_event at OBEX_EV_REQ");
 					}
 					break;
 				case OBEX_CMD_PUT:
 					g_message ("CMD_PUT requested");
-					if (!server_session->priv->allow_write) {
-						g_message ("CMD_PUT forbidden");
-						OBEX_ObjectSetRsp (object, OBEX_RSP_FORBIDDEN, 
-											OBEX_RSP_FORBIDDEN);
-						return;
-					}
-					/* Check if we have write permissions for current path */
-					if (g_access (server_session->priv->path, W_OK) < 0) {
-						g_message ("No write permissions for current path");
-						OBEX_ObjectSetRsp (object, OBEX_RSP_UNAUTHORIZED, 
-											OBEX_RSP_UNAUTHORIZED);
-						return;
-					}
-					ret = ods_obex_srv_put (obex_context, object, 
-											server_session->priv->path);
-					g_message ("ret=%d", ret);
-					if (ret == 0 && obex_context->report_progress) {
-						g_signal_emit (server_session, signals [TRANSFER_STARTED],
-										0, obex_context->remote,
-										obex_context->local,
-										obex_context->target_size);
-						obex_context->transfer_started_signal_emitted = TRUE;
-					}
+					obex_request_put (server_session, obex_context, object);
 					break;
 				default:
 					OBEX_ObjectSetRsp (object, OBEX_RSP_NOT_IMPLEMENTED, 
@@ -387,38 +569,45 @@ obex_event (obex_t *handle, obex_object_t *object, int mode, int event,
 			obex_transfer_data_exchange_done (server_session, ret);
 			break;
 		case OBEX_EV_STREAMAVAIL:
+			ret = 0;
+			
 			if (!server_session->priv->allow_write) {
 				g_message ("CMD_PUT forbidden");
 				OBEX_ObjectSetRsp (object, OBEX_RSP_FORBIDDEN, 
 									OBEX_RSP_FORBIDDEN);
+				/* We don't emit ErrorOccurred for this situation */
 				return;
 			}
 			/* Check if we have write permissions for current path */
 			if (g_access (server_session->priv->path, W_OK) < 0) {
-				g_message ("No write permissions for current path");
 				OBEX_ObjectSetRsp (object, OBEX_RSP_UNAUTHORIZED, 
 									OBEX_RSP_UNAUTHORIZED);
-				return;
+				ret = -EACCES;
+				goto readstream_done;
 			}
-			ret = 0;
-						
 			/* This PUT request is definitely not a delete request. Let's
-			 * open a file for writing. */
+			 * open a file for writing in case this wasn't done yet */
 			if (obex_context->remote && !obex_context->local) {
-				obex_context->report_progress = TRUE;
-				if (!ods_obex_srv_new_file (obex_context, server_session->priv->path))
-					ret = -1;
-			}
-			if (ret == 0)
-				ret = ods_obex_readstream (obex_context, object);
-			/* only suspend if we have transfer info */
-			if (!server_session->priv->auto_accept && obex_context->local &&
-					!obex_context->suspended_once) {
-				g_message ("suspending request");
-				obex_context->suspended_once = TRUE;
-				OBEX_SuspendRequest (obex_context->obex_handle, object);
+				if (!ods_obex_srv_new_file (obex_context, server_session->priv->path)) {
+					OBEX_ObjectSetRsp (object, OBEX_RSP_UNAUTHORIZED, 
+										OBEX_RSP_UNAUTHORIZED);
+					ret = -EACCES;
+					goto readstream_done;
+				}
 			}
+			/* suspend request if necessary */
+			obex_suspend_request (server_session, obex_context);
+			ret = ods_obex_readstream (obex_context, object);
+readstream_done:
 			obex_transfer_data_exchange_done (server_session, ret);
+			if (ret == 2) {
+				/* Transfer was cancelled by sending RSP_FORBIDDEN, this will
+				 * not trigger EV_ABORT so we need to emit Cancelled signal here */
+				obex_request_cancelled (server_session, obex_context);
+			} else if (ret == 1) {
+				/* Last packet was received, send image handle */
+				send_image_handle (server_session, obex_context, object);
+			}
 			break;
 		case OBEX_EV_PARSEERR:
 			/* Handled in io_callback */
@@ -430,6 +619,48 @@ obex_event (obex_t *handle, obex_object_t *object, int mode, int event,
 	}
 }
 
+static void
+setup_transport (OdsServerSession *server_session)
+{
+	OdsObexContext *obex_context;
+	GIOChannel *chan;
+	gint ret;
+	/*GError *error = NULL;*/
+	g_message ("Creating server session");
+	
+	obex_context = server_session->priv->obex_context;
+	
+	/* call OBEX_Init, setup FD Transport here */
+	obex_context->obex_handle = OBEX_Init (OBEX_TRANS_FD, obex_event, 0);
+	if (obex_context->obex_handle == NULL) {
+		/* error (out of memory) */
+		g_warning ("error out of memory");
+		return;
+	}
+	OBEX_SetUserData (obex_context->obex_handle, server_session);
+
+	OBEX_SetTransportMTU (obex_context->obex_handle, 
+							ODS_OBEX_RX_MTU, 
+							ODS_OBEX_TX_MTU);
+
+	ret = FdOBEX_TransportSetup (obex_context->obex_handle, 
+									server_session->priv->fd, 
+									server_session->priv->fd, 
+									0);
+	if (ret < 0) {
+		OBEX_Cleanup (obex_context->obex_handle);
+		/* error (transport error or smth) */
+		g_warning ("error transport setup fail");
+		return;
+	}
+	chan = g_io_channel_unix_new (server_session->priv->fd);
+	g_io_channel_set_close_on_unref (chan, TRUE);
+	server_session->priv->io_watch = g_io_add_watch(chan,
+							G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
+							obex_io_callback, obex_context->obex_handle);
+	g_io_channel_unref(chan);
+}
+
 static void
 ods_server_session_set_property (GObject      *object,
                         			guint         property_id,
@@ -441,6 +672,8 @@ ods_server_session_set_property (GObject      *object,
 	switch (property_id) {
 		case ODS_SERVER_SESSION_FD:
 			self->priv->fd = g_value_get_int (value);
+			if (self->priv->fd >= 0)
+				setup_transport (self);
 			break;
 		case ODS_SERVER_SESSION_SERVICE:
 			self->priv->service = g_value_get_int (value);
@@ -456,6 +689,9 @@ ods_server_session_set_property (GObject      *object,
 		case ODS_SERVER_SESSION_AUTO_ACCEPT:
 			self->priv->auto_accept = g_value_get_boolean (value);
 			break;
+		case ODS_SERVER_SESSION_REQUIRE_IMAGING_THUMBNAILS:
+			self->priv->require_imaging_thumbnails = g_value_get_boolean (value);
+			break;
 		case ODS_SERVER_SESSION_OWNER:
 			self->priv->owner = g_value_dup_string (value);
 			break;
@@ -490,6 +726,9 @@ ods_server_session_get_property (GObject      *object,
 		case ODS_SERVER_SESSION_AUTO_ACCEPT:
 			g_value_set_boolean (value, self->priv->auto_accept);
 			break;
+		case ODS_SERVER_SESSION_REQUIRE_IMAGING_THUMBNAILS:
+			g_value_set_boolean (value, self->priv->require_imaging_thumbnails);
+			break;
 		case ODS_SERVER_SESSION_OWNER:
 			g_value_set_string (value, self->priv->owner);
 			break;
@@ -512,7 +751,6 @@ ods_server_session_class_init (OdsServerSessionClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	
-	object_class->constructor = ods_server_session_constructor;
 	object_class->finalize = ods_server_session_finalize;
 	
 	object_class->set_property = ods_server_session_set_property;
@@ -522,9 +760,9 @@ ods_server_session_class_init (OdsServerSessionClass *klass)
 									ODS_SERVER_SESSION_FD,
 									g_param_spec_int ("fd",
 										"", "",
-										0, G_MAXINT, /* min, max values */
+										-1, G_MAXINT, /* min, max values */
 										0 /* default value */,
-										G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
+										G_PARAM_READWRITE));
 										
 	g_object_class_install_property (object_class,
 									ODS_SERVER_SESSION_SERVICE,
@@ -561,6 +799,13 @@ ods_server_session_class_init (OdsServerSessionClass *klass)
 										"", "",
 										TRUE,
 										G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
+	
+	g_object_class_install_property (object_class,
+									ODS_SERVER_SESSION_REQUIRE_IMAGING_THUMBNAILS,
+									g_param_spec_boolean("require-imaging-thumbnails",
+										"", "",
+										FALSE,
+										G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
 										
 	g_object_class_install_property (object_class,
 									ODS_SERVER_SESSION_DBUS_PATH,
@@ -623,6 +868,15 @@ ods_server_session_class_init (OdsServerSessionClass *klass)
 			      NULL,
 			      ods_marshal_VOID__STRING_STRING,
 			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_STRING);
+	signals [REMOTE_DISPLAY_REQUESTED] =
+		g_signal_new ("remote-display-requested",
+				  G_TYPE_FROM_CLASS (object_class),
+				  G_SIGNAL_RUN_LAST,
+				  G_STRUCT_OFFSET (OdsServerSessionClass, remote_display_requested),
+				  NULL,
+				  NULL,
+				  g_cclosure_marshal_VOID__STRING,
+				  G_TYPE_NONE, 1, G_TYPE_STRING);
 	
 	g_type_class_add_private (klass, sizeof (OdsServerSessionPrivate));
 	
@@ -656,12 +910,11 @@ ods_server_session_init (OdsServerSession *server_session)
 	
 	/* figure out DBus object path for this instance */
 	server_session->priv->state = ODS_SERVER_SESSION_STATE_OPEN;
-	server_session->priv->dbus_path = (gchar *)g_malloc0 (
-										ODS_SERVER_SESSION_DBUS_PATH_MAX_LENGTH);
-	g_sprintf (server_session->priv->dbus_path, 
-				ODS_SERVER_SESSION_DBUS_PATH_PATTERN, 
-				iterator);
+	server_session->priv->dbus_path = g_strdup_printf (ODS_SERVER_SESSION_DBUS_PATH_PATTERN,
+														iterator);
 	iterator++;
+	/* default selected image handle for BIP RemoteDisplay is 0000001 */
+	server_session->priv->selected_img = 1;
 	
 	/* create mutex */
 	g_static_mutex_init (&server_session->priv->mutex);
@@ -671,58 +924,6 @@ ods_server_session_init (OdsServerSession *server_session)
 							G_OBJECT (server_session));
 }
 
-static GObject*
-ods_server_session_constructor (GType type, guint n_construct_params, 
-								GObjectConstructParam *construct_params)
-{
-	GObject *object;
-	OdsServerSession *server_session;
-	OdsObexContext *obex_context;
-	GIOChannel *chan;
-	gint ret;
-	/*GError *error = NULL;*/
-	g_message ("Creating server session");
-	
-	object = G_OBJECT_CLASS (ods_server_session_parent_class)->constructor (type,
-                                                           n_construct_params,
-                                                           construct_params);
-	
-	server_session = ODS_SERVER_SESSION (object);
-	obex_context = server_session->priv->obex_context;
-	
-	/* call OBEX_Init, setup FD Transport here */
-	obex_context->obex_handle = OBEX_Init (OBEX_TRANS_FD, obex_event, 0);
-	if (obex_context->obex_handle == NULL) {
-		/* error (out of memory) */
-		g_warning ("error out of memory");
-		return NULL;
-	}
-	OBEX_SetUserData (obex_context->obex_handle, server_session);
-
-	OBEX_SetTransportMTU (obex_context->obex_handle, 
-							ODS_OBEX_RX_MTU, 
-							ODS_OBEX_TX_MTU);
-
-	ret = FdOBEX_TransportSetup (obex_context->obex_handle, 
-									server_session->priv->fd, 
-									server_session->priv->fd, 
-									0);
-	if (ret < 0) {
-		OBEX_Cleanup (obex_context->obex_handle);
-		/* error (transport error or smth) */
-		g_warning ("error transport setup fail");
-		return NULL;
-	}
-	chan = g_io_channel_unix_new (server_session->priv->fd);
-	g_io_channel_set_close_on_unref (chan, TRUE);
-	server_session->priv->io_watch = g_io_add_watch(chan,
-							G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
-							obex_io_callback, obex_context->obex_handle);
-	g_io_channel_unref(chan);
-	
-	return object;
-}
-
 /**
  * ods_server_session_finalize:
  * @object: The object to finalize
@@ -749,6 +950,8 @@ ods_server_session_finalize (GObject *object)
 	g_free (server_session->priv->obex_context);
 
 	/* free other private variables */
+	if (server_session->priv->img_handle_list)
+		g_hash_table_unref (server_session->priv->img_handle_list);
 	g_free (server_session->priv->root_path);
 	g_free (server_session->priv->path);
 	g_free (server_session->priv->owner);
@@ -766,6 +969,7 @@ ods_server_session_finalize (GObject *object)
 OdsServerSession *
 ods_server_session_new (gint fd, gint service, const gchar *path, 
 						gboolean allow_write, gboolean auto_accept,
+						gboolean require_imaging_thumbnails,
 						const gchar *owner)
 {
 	OdsServerSession *server_session;
@@ -775,6 +979,7 @@ ods_server_session_new (gint fd, gint service, const gchar *path,
 									"path", path,
 									"allow-write", allow_write,
 									"auto-accept", auto_accept,
+									"require-imaging-thumbnails", require_imaging_thumbnails,
 									"owner", owner,
 									NULL);
 	return ODS_SERVER_SESSION (server_session);
@@ -802,7 +1007,7 @@ ods_server_session_accept (OdsServerSession *server_session,
 	}
 	
 	/* Accept file */
-	OBEX_ResumeRequest (server_session->priv->obex_context->obex_handle);
+	server_session->priv->obex_context->suspend_result = OBEX_SUSP_ACCEPTED;
 	
 	dbus_g_method_return (context);
 	ODS_SERVER_SESSION_UNLOCK (server_session);
@@ -831,7 +1036,7 @@ ods_server_session_reject (OdsServerSession *server_session,
 	}
 	
 	/* Reject file */
-	ods_server_session_cancel_internal (server_session);
+	server_session->priv->obex_context->suspend_result = OBEX_SUSP_REJECTED;
 	
 	dbus_g_method_return (context);
 	ODS_SERVER_SESSION_UNLOCK (server_session);
@@ -845,7 +1050,6 @@ ods_server_session_disconnect_internal (OdsServerSession *server_session)
 		OBEX_TransportDisconnect (server_session->priv->obex_context->obex_handle);
 		server_session->priv->state = ODS_SERVER_SESSION_STATE_NOT_CONNECTED;
 	}
-	g_source_remove (server_session->priv->io_watch);
 	g_idle_add ((GSourceFunc) emit_disconnected, server_session);
 }
 
@@ -880,45 +1084,30 @@ ods_server_session_disconnect (OdsServerSession *server_session,
 GHashTable *
 ods_server_session_get_transfer_info (OdsServerSession *server_session)
 {
-	GHashTable *info;
-	
-	gchar *time_str = (gchar *)g_malloc (17);
-	
-	info = g_hash_table_new ((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal);
-	g_hash_table_insert (info, "LocalPath", 
-							g_strdup (server_session->priv->obex_context->local));
-	g_hash_table_insert (info, "RemoteFilename",
-							g_strdup (server_session->priv->obex_context->remote));
-	g_hash_table_insert (info, "Size",
-							g_strdup_printf ("%" G_GUINT64_FORMAT, 
-								server_session->priv->obex_context->target_size));
-	if (server_session->priv->obex_context->modtime != -1)
-		ods_make_iso8601 (server_session->priv->obex_context->modtime, time_str, 
-							sizeof (time_str));
-	else
-		time_str = "";
-	g_hash_table_insert (info, "Time", time_str);
-	return info;
+	return ods_obex_transfer_get_info (server_session->priv->obex_context);
 }
 
-gint
+gboolean
 ods_server_session_cancel_internal (OdsServerSession *server_session)
 {
 	if (server_session->priv->state != ODS_SERVER_SESSION_STATE_BUSY) {
 		/* emit CANCELLED signal now */
 		g_signal_emit (server_session, signals[CANCELLED], 0);
-		return 1;
+		return FALSE;
 	}
-	/* Send CMD_ABORT; cleanup will be done in obex_event */
-	return OBEX_CancelRequest (server_session->priv->obex_context->obex_handle, TRUE);
+	/* Send CMD_ABORT at obex_writestream/readstream; 
+	 * cleanup will be done in obex_event */
+	server_session->priv->obex_context->cancelled = TRUE;
+	/* In case ServerSession got stuck in suspended mode (client app quitted without
+	 * accepting/rejecting transfer), we reject transfer at this point */
+	server_session->priv->obex_context->suspend_result = OBEX_SUSP_REJECTED;
+	return TRUE;
 }
 
 gboolean
 ods_server_session_cancel (OdsServerSession *server_session,
 							DBusGMethodInvocation *context)
 {
-	GError *error = NULL;
-	
 	ODS_SERVER_SESSION_LOCK (server_session);
 	/* do checks */
 	if (!ods_check_caller (context, server_session->priv->owner)) {
@@ -926,16 +1115,24 @@ ods_server_session_cancel (OdsServerSession *server_session,
 		return FALSE;
 	}
 	
-	if (ods_server_session_cancel_internal (server_session) == -1) {
-		g_set_error (&error, ODS_ERROR, ODS_ERROR_OUT_OF_MEMORY, "Out of memory");
-		dbus_g_method_return_error (context, error);
-		g_clear_error (&error);
-		ODS_SERVER_SESSION_UNLOCK (server_session);
-	} else {
+	if (ods_server_session_cancel_internal (server_session)) {
+		if (server_session->priv->dbus_context) {
+			GError *error;
+			
+			g_set_error (&error, ODS_ERROR, ODS_ERROR_FAILED,
+							"DBus context is set (this is probably a bug)");
+			dbus_g_method_return_error (context, error);
+			g_clear_error (&error);
+			ODS_SERVER_SESSION_UNLOCK (server_session);
+			return FALSE;
+		}
 		/* set dbus context */
-		g_assert (!server_session->priv->dbus_context);
 		server_session->priv->dbus_context = context;
 		/* will return at obex_event{EV_ABORT} */
+	} else {
+		dbus_g_method_return (context);
+		ODS_SERVER_SESSION_UNLOCK (server_session);
 	}
+	
 	return TRUE;
 }
