@@ -29,10 +29,8 @@
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #include <utime.h>
-#endif /* HAVE_UNISTD_H */
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/rfcomm.h>
@@ -59,6 +57,14 @@ static void     ods_server_finalize	(GObject		*object);
 
 #define ODS_SERVER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ODS_TYPE_SERVER, OdsServerPrivate))
 
+/* this is used for timed assignment of ServerSession fd */
+#define SERVER_SESSION_INIT_TIMEOUT 250 /* timeout in milliseconds */
+typedef struct OdsServerSessionCbData_ {
+	gint				fd;
+	OdsServerSession	*session;
+} OdsServerSessionCbData;
+
+/* this structure is used for ServerSession list */
 typedef struct OdsServerSessionInfo_ {
 	OdsServerSession	*session;
 	/* Bluetooth specific */
@@ -77,6 +83,9 @@ struct OdsServerPrivate
 	gint					fd; /* rfcomm device */
 	guint					service;
 	gchar					*owner; /* D-Bus client, who initiated this server */
+	/* additional options (set via SetOption) */
+	guint					limit;
+	gboolean				require_imaging_thumbnails;
 	/* io channel */
 	GIOChannel				*io_channel;
 	GSource					*io_source;
@@ -100,6 +109,9 @@ enum {
 	LAST_SIGNAL
 };
 
+#define OPT_REQ_IMG_THUMBNAILS	"RequireImagingThumbnails"
+#define OPT_LIMIT				"Limit"
+
 static guint	signals [LAST_SIGNAL] = { 0, };
 /* for numbering servers */
 static guint	iterator = 0;
@@ -172,6 +184,13 @@ server_session_cancelled_cb (OdsServerSession *server_session)
 	ods_server_session_disconnect_internal (server_session);
 }
 
+static gboolean
+server_session_cancel (OdsServerSession *server_session)
+{
+	ods_server_session_cancel_internal (server_session);
+	return FALSE;
+}
+
 static gboolean
 server_session_finalize (gpointer key, OdsServerSessionInfo *session_info,
 							OdsServer *server)
@@ -179,7 +198,7 @@ server_session_finalize (gpointer key, OdsServerSessionInfo *session_info,
 	g_message ("attempting to close server session");
 	g_signal_connect (session_info->session, "cancelled", 
 						G_CALLBACK (server_session_cancelled_cb), NULL);
-	g_idle_add ((GSourceFunc) ods_server_session_cancel_internal, session_info->session);
+	g_idle_add ((GSourceFunc) server_session_cancel, session_info->session);
 	/* Even if there was nothing to cancel, we will get
 	 * CANCELLED signal and disconnection will happen in 
 	 * session_cancelled_cb */
@@ -187,6 +206,16 @@ server_session_finalize (gpointer key, OdsServerSessionInfo *session_info,
 	return TRUE;
 }
 
+static gboolean
+server_session_fd_assign (OdsServerSessionCbData *data)
+{
+	OdsServerSession *session = data->session;
+	
+	g_object_set (session, "fd", data->fd, NULL);
+	g_free (data);
+	return FALSE;
+}
+
 static gboolean
 connect_callback (GIOChannel *source, GIOCondition cond, gpointer data)
 {
@@ -196,6 +225,7 @@ connect_callback (GIOChannel *source, GIOCondition cond, gpointer data)
 	OdsServer			*server;
 	OdsServerSession	*session;
 	gchar				*session_object;
+	OdsServerSessionCbData *cb_data;
 	GError				*error = NULL;
 	gboolean			ret = TRUE;
 	
@@ -214,9 +244,18 @@ connect_callback (GIOChannel *source, GIOCondition cond, gpointer data)
 	size = sizeof (struct sockaddr_rc);
 	cli_fd = accept (srv_fd, (struct sockaddr*) &cli_addr, &size);
 	if (!server->priv->started) {
-		shutdown (cli_fd, SHUT_RDWR);		
+		shutdown (cli_fd, SHUT_RDWR);	
+		close (cli_fd);
 		return TRUE; /* ignore requests until server is started */
 	}
+	if (server->priv->limit > 0 &&
+			g_hash_table_size (server->priv->session_list) == server->priv->limit) {
+		g_message ("ServerSession limit reached (%d), refusing connection",
+					server->priv->limit);
+		shutdown (cli_fd, SHUT_RDWR);
+		close (cli_fd);
+		return TRUE;
+	}
 	if (cli_fd < 0) {
 		g_message ("Error while accepting connection: %s", g_strerror (errno));
 		ods_error_err2gerror (errno, &error);
@@ -224,23 +263,22 @@ connect_callback (GIOChannel *source, GIOCondition cond, gpointer data)
 		goto out;
 	}
 
-    /*if (g_slist_length (server->priv->session_list) >= server->limit) {
-        debug("Maximum client limit (%d) reached: refusing connect attempt",
-                server->limit);
-        close(cli_fd);
-        return TRUE;
-    }*/
-
-	session = ods_server_session_new (cli_fd, server->priv->service,
+	/* To avoid race condition when client application misses TransferStarted
+	 * signal from ServerSession object, we emit SessionCreated signal now,
+	 * but assign fd to ServerSession (and therefore start OBEX operations)
+	 * only after a timeout */
+	session = ods_server_session_new (-1, server->priv->service,
 										server->priv->path,
 										server->priv->allow_write,
 										server->priv->auto_accept,
+										server->priv->require_imaging_thumbnails,
 										server->priv->owner);
-    if (!session) {
-    	g_set_error (&error, ODS_ERROR, ODS_ERROR_FAILED, "Could not create OdsServerSession object");
-        goto out;
-    }
-
+	cb_data = g_new0 (OdsServerSessionCbData, 1);
+	cb_data->fd = cli_fd;
+	cb_data->session = session;
+	g_timeout_add (SERVER_SESSION_INIT_TIMEOUT,
+					(GSourceFunc) server_session_fd_assign, cb_data);
+	
 	/* deal with signals */
 	g_signal_connect (session, "disconnected",
 						G_CALLBACK (server_session_disconnected_cb), server);
@@ -554,6 +592,7 @@ ods_server_finalize (GObject *object)
 	g_source_destroy (server->priv->io_source);
 	g_io_channel_shutdown (server->priv->io_channel, TRUE, NULL);
 	g_io_channel_unref(server->priv->io_channel);
+	close (server->priv->fd);
 	/* free other private variables */
 	g_free (server->priv->owner);
 	g_free (server->priv->dbus_path);
@@ -704,6 +743,47 @@ ods_server_is_started (OdsServer *server)
 	return server->priv->started;
 }
 
+gboolean
+ods_server_set_option (OdsServer *server, const gchar *name, GValue *value,
+						DBusGMethodInvocation *context)
+{
+	GError *error = NULL;
+	
+	/* Check caller */
+	if (!ods_check_caller (context, server->priv->owner))
+		return FALSE;
+	if (!g_ascii_strncasecmp (name, OPT_REQ_IMG_THUMBNAILS, strlen (name))) {
+		/* require-imaging-thumbnails */
+		if (!G_VALUE_HOLDS_BOOLEAN (value)) {
+			g_set_error (&error, ODS_ERROR, ODS_ERROR_INVALID_ARGUMENTS,
+							"Invalid value for require-imaging-thumbnails");
+			goto out;
+		}
+		server->priv->require_imaging_thumbnails = g_value_get_boolean (value);
+	} else if (!g_ascii_strncasecmp (name, OPT_LIMIT, strlen (name))) {
+		/* limit */
+		if (!G_VALUE_HOLDS_UINT (value)) {
+			g_set_error (&error, ODS_ERROR, ODS_ERROR_INVALID_ARGUMENTS,
+							"Invalid value for limit");
+			goto out;
+		}
+		server->priv->limit = g_value_get_uint (value);
+	} else {
+		g_set_error (&error, ODS_ERROR, ODS_ERROR_INVALID_ARGUMENTS,
+						"Unknown option");
+		goto out;
+	}
+	
+out:
+	if (error) {
+		dbus_g_method_return_error (context, error);
+		g_clear_error (&error);
+		return FALSE;
+	}
+	dbus_g_method_return (context);
+	return TRUE;
+}
+
 GHashTable *
 ods_server_get_server_session_info (OdsServer *server, gchar *session_object)
 {
@@ -718,3 +798,9 @@ ods_server_get_server_session_info (OdsServer *server, gchar *session_object)
 	}
 	return info;
 }
+
+gchar**
+ods_server_get_server_session_list (OdsServer *server)
+{
+	return ods_hash_table_keys2strv (server->priv->session_list);
+}
