@@ -27,16 +27,6 @@
 
 G_BEGIN_DECLS
 
-#define	ODS_SESSION_DBUS_PATH_PATTERN		"/org/openobex/session%d"
-#define ODS_SESSION_DBUS_PATH_MAX_LENGTH	31
-#define	ODS_SESSION_DBUS_INTERFACE			"org.openobex.Session"
-
-typedef enum {
-	ODS_SESSION_STATE_NOT_CONNECTED,
-	ODS_SESSION_STATE_OPEN,
-	ODS_SESSION_STATE_BUSY
-} OdsSessionState;
-
 #define ODS_TYPE_SESSION	 (ods_session_get_type ())
 #define ODS_SESSION(o)		 (G_TYPE_CHECK_INSTANCE_CAST ((o), ODS_TYPE_SESSION, OdsSession))
 #define ODS_SESSION_CLASS(k)	 (G_TYPE_CHECK_CLASS_CAST((k), ODS_TYPE_SESSION, OdsSessionClass))
@@ -48,7 +38,11 @@ enum {
   ODS_SESSION_FD = 1,
   ODS_SESSION_SERVICE,
   ODS_SESSION_OWNER,
-  ODS_SESSION_DBUS_PATH
+  ODS_SESSION_DBUS_PATH,
+  ODS_SESSION_IMAGING_FEATURE,
+  ODS_SESSION_IMAGING_SDP_DATA,
+  ODS_SESSION_TARGET_UUID,
+  ODS_SESSION_USBINTFNUM
 };
 
 typedef struct OdsSessionPrivate OdsSessionPrivate;
@@ -63,7 +57,9 @@ typedef struct
 {
 	GObjectClass	parent_class;
 	void		(* cancelled)			(OdsSession	*session);
-	void		(* connected)			(OdsSession	*session);
+	void		(* connect_result_internal)	(OdsSession	*session,
+											const gchar *error_name,
+											const gchar *error_message);
 	void		(* disconnected)		(OdsSession *session);
 	void		(* closed)				(OdsSession *session);
 	void		(* transfer_started)	(OdsSession *session,
@@ -76,23 +72,26 @@ typedef struct
 	void		(* error_occurred)		(OdsSession *session,
 											const gchar *error_name,
 											const gchar *error_message);
+	void		(* image_handle_received)(OdsSession *session,
+											const gchar *image_handle,
+											gboolean thumbnail_requested);
 	DBusGConnection *connection;
 } OdsSessionClass;
 
 
 GType		 ods_session_get_type		  				(void);
 OdsSession	*ods_session_new							(gint fd,
+															gint usb_interface_number,
 															gint service,
-															const gchar *owner);
-gboolean	 ods_session_connect						(OdsSession *session,
-															DBusGMethodInvocation *context);
+															const gchar *owner,
+															const gchar *target_uuid);
+gboolean	 ods_process_usb_input						(gpointer *data);
 gint		 ods_session_disconnect_internal			(OdsSession *session,
 															GError **error);
 gboolean	 ods_session_disconnect						(OdsSession *session,
 															DBusGMethodInvocation *context);
 gboolean	 ods_session_close							(OdsSession *session,
 															DBusGMethodInvocation *context);
-gboolean	 ods_session_is_connected					(OdsSession *session);
 gboolean	 ods_session_change_current_folder			(OdsSession *session,
 															const gchar *path,
 															DBusGMethodInvocation *context);
@@ -105,6 +104,10 @@ gboolean	 ods_session_copy_remote_file				(OdsSession *session,
 															const gchar *remote_filename,
 															const gchar *local_path,
 															DBusGMethodInvocation *context);
+gboolean	 ods_session_copy_remote_file_by_type 		(OdsSession *session,
+															const gchar *type,
+															const gchar *local_path,
+															DBusGMethodInvocation *context);
 gboolean	 ods_session_create_folder					(OdsSession *session,
 															const gchar *folder_name,
 															DBusGMethodInvocation *context);
@@ -112,15 +115,66 @@ gboolean	 ods_session_retrieve_folder_listing		(OdsSession *session,
 															DBusGMethodInvocation *context);
 gboolean	 ods_session_get_capability					(OdsSession *session,
 															DBusGMethodInvocation *context);
+gboolean	 ods_session_get_imaging_capabilities		(OdsSession *session,
+															DBusGMethodInvocation *context);
+gboolean	 ods_session_send_file_ext					(OdsSession *session,
+															const gchar *local_path,
+															const gchar *remote_filename,
+															const gchar *type,
+															DBusGMethodInvocation *context);
 gboolean	 ods_session_send_file						(OdsSession *session,
 															const gchar *local_path,
 															DBusGMethodInvocation *context);
+gboolean	 ods_session_set_transfer_hints				(OdsSession *session,
+															const gchar *fifo,
+															const gchar *remote_filename,
+															const gchar *type,
+															guint64 size,
+															gint64 mtime,
+															gint64 ctime,
+															DBusGMethodInvocation *context);
 gboolean	 ods_session_delete_remote_file				(OdsSession *session,
 															const gchar *remote_filename,
 															DBusGMethodInvocation *context);
+gboolean	 ods_session_remote_copy				(OdsSession *session,
+															const gchar *remote_source,
+															const gchar *remote_destination,
+															DBusGMethodInvocation *context);
+gboolean	 ods_session_remote_move				(OdsSession *session,
+															const gchar *remote_source,
+															const gchar *remote_destination,
+															DBusGMethodInvocation *context);
+gboolean	 ods_session_get_image_info					(OdsSession *session,
+															const gchar *local_path,
+															DBusGMethodInvocation *context);
+gboolean	 ods_session_put_image						(OdsSession *session,
+															const gchar *local_path,
+															DBusGMethodInvocation *context);
+gboolean	 ods_session_put_image_resized				(OdsSession *session,
+															const gchar *local_path,
+															guint16 width,
+															guint16 height,
+															const gchar *encoding,
+															const gchar *transformation,
+															DBusGMethodInvocation *context);
+gboolean	 ods_session_put_linked_attachment			(OdsSession *session,
+															const gchar *image_handle,
+															const gchar *local_path,
+															const gchar *content_type,
+															const gchar *charset,
+															DBusGMethodInvocation *context);
+gboolean	 ods_session_remote_display_select_image	(OdsSession *session,
+															const gchar *image_handle,
+															DBusGMethodInvocation *context);
+gboolean	 ods_session_remote_display_show_current_image (OdsSession *session,
+															DBusGMethodInvocation *context);
+gboolean	 ods_session_remote_display_show_next_image (OdsSession *session,
+															DBusGMethodInvocation *context);
+gboolean	 ods_session_remote_display_show_previous_image (OdsSession *session,
+															DBusGMethodInvocation *context);
 GHashTable	*ods_session_get_transfer_info				(OdsSession *session);
 gboolean	 ods_session_is_busy						(OdsSession *session);
-gint		 ods_session_cancel_internal				(OdsSession *session);
+gboolean	 ods_session_cancel_internal				(OdsSession *session);
 gboolean	 ods_session_cancel							(OdsSession *session,
 															DBusGMethodInvocation *context);
 G_END_DECLS
