@@ -4,16 +4,11 @@ import dbus
 import dbus.decorators
 import dbus.glib
 import gobject
+from optparse import OptionParser
 import sys
 from signal import *
 import time
 
-# enable server on all adapters
-bt_address = '00:00:00:00:00:00'
-allow_write = True
-auto_accept = True
-accept = True
-
 class ServerSession:
     
     def __init__(self, session_object_path):
@@ -31,6 +26,7 @@ class ServerSession:
         self.session.connect_to_signal('TransferProgress', self.transfer_progress_cb)
         self.session.connect_to_signal('TransferCompleted', self.transfer_completed_cb)
         self.session.connect_to_signal('ErrorOccurred', self.error_occurred_cb)
+        self.session.connect_to_signal('RemoteDisplayRequested', self.remote_display_requested_cb)
     
     # emitted when session is disconnected
     def disconnected_cb(self):
@@ -47,11 +43,16 @@ class ServerSession:
         print self.prefix,
         print 'Transfer started (%s, %s, %d)' % (filename, local_path, total_bytes)
         self.total_bytes = total_bytes
-        if auto_accept == False:
-            print self.prefix, 'Sleeping for 3 secs'
-            time.sleep(3)
-            print self.prefix,
-            if accept == True:
+        
+        info = self.session.GetTransferInfo()
+        print self.prefix, 'All transfer info:'
+        for name,value in info.iteritems():
+            print self.prefix, '--', name, '=', value
+        
+        if options.ask_to_accept:
+            print "Accept file? Type 'a' to accept, 'r' to reject:"
+            command = raw_input('>>> ')
+            if command.strip() == 'a':
                 print 'Accepting'
                 self.session.Accept()
             else:
@@ -61,7 +62,7 @@ class ServerSession:
     # emitted constantly during transfer
     def transfer_progress_cb(self, bytes_transferred):
         print self.prefix,
-        if self.total_bytes != -1:
+        if self.total_bytes > 0:
             print 'Progress: %d %%' % int(float(bytes_transferred)/self.total_bytes*100)
         else:
             print 'Progress'
@@ -75,13 +76,19 @@ class ServerSession:
     def error_occurred_cb(self, error_name, error_message):
         print self.prefix,
         print 'Error occurred: %s: %s' % (error_name, error_message)
+        
+    # emitted when BIP RemoteDisplay feature is requested
+    def remote_display_requested_cb(self, img_filename):
+   	    print self.prefix,
+   	    print 'RemoteDisplay requested for image: ', img_filename
 
 
 class Tester:
     
-    def __init__(self, server_type, root_path):
+    def __init__(self, server_type, root_path, options):
         self.server_type = server_type
         self.root_path = root_path
+        self.options = options
         
         # get Session bus
         self.bus = dbus.SessionBus()
@@ -91,10 +98,12 @@ class Tester:
         self.manager = dbus.Interface(manager_obj, 'org.openobex.Manager')
         
         # call CreateBluetoothServer with specified server type 
-        # (opp - Object Push Profile, ftp - File Transfer Profile)
+        # (opp - Object Push Profile,
+        #  ftp - File Transfer Profile,
+        #  bip - Basic Imaging Profile)
         # returns Server object path
-        require_pairing = (self.server_type == 'ftp')
-        server_path = self.manager.CreateBluetoothServer(bt_address, self.server_type, require_pairing)
+        server_path = self.manager.CreateBluetoothServer(options.local_device,
+                                             self.server_type, options.pairing)
         print 'Server object: ', server_path
         # get org.openobex.Server object
         server_obj = self.bus.get_object('org.openobex', server_path)
@@ -108,8 +117,10 @@ class Tester:
         self.server.connect_to_signal('SessionCreated', self.session_created_cb)
         self.server.connect_to_signal('SessionRemoved', self.session_removed_cb)
         
+        # require remote device to send thumbnails for BIP PutImage sessions
+        self.server.SetOption('RequireImagingThumbnails', options.thumbnails)
         # start server with specified options
-        self.server.Start(self.root_path, allow_write, auto_accept)
+        self.server.Start(self.root_path, not options.readonly, not options.ask_to_accept)
         
         self.main_loop = gobject.MainLoop()
         self.main_loop.run()
@@ -144,5 +155,32 @@ if __name__ == '__main__':
     gobject.threads_init()
     dbus.glib.init_threads()
 
-    tester = Tester(sys.argv[1], sys.argv[2])
+    usage = 'Usage: '+sys.argv[0]+' [options] profile path'
+    parser = OptionParser(usage)
+    parser.add_option('-l', '--local', dest='local_device',
+                      default='00:00:00:00:00:00',
+                      help='ADDRESS of Bluetooth adapter to listen on. Default is 00:00:00:00:00:00',
+                      metavar='ADDRESS')
+    parser.add_option('-p', '--pairing', dest='pairing',
+                      action='store_true', default=False,
+                      help='Require remote devices to be paired before allowing them to connect. Disabled by default')
+    parser.add_option('-r', '--readonly', dest='readonly',
+                      action='store_true', default=False,
+                      help='Disallow any write operations. Allowed by default')
+    parser.add_option('-t', '--thumbnails', dest='thumbnails',
+                      action='store_true', default=False,
+                      help='Require remote device to send thumbnails when using Imaging (BIP) server')
+    parser.add_option('-a', '--ask-to-accept', dest='ask_to_accept',
+                      action='store_true', default=False,
+                      help='Prompt user to accept or reject every file. By default all files are accepted')
+                      
+    options, args = parser.parse_args()
+    
+    if len(args) != 2:
+        print usage
+        print
+        print 'error: Wrong number of arguments'
+        exit()
+    
+    tester = Tester(args[0], args[1], options)
     
